import drf_turbo

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.viewcode",
    "sphinx.ext.intersphinx",
    "sphinx_issues",
]

primary_domain = "py"
default_role = "py:obj"

github_user = "Mng-dev-ai"
github_repo = "drf-turbo"

issues_github_path = f"{github_user}/{github_repo}"

# The master toctree document.
master_doc = "index"
language = "en"
html_domain_indices = False
source_suffix = ".rst"
project = "drf-turbo"
copyright = "2021, Michael Gendy"
version = release = drf_turbo.__version__
templates_path = ["_templates"]
exclude_patterns = ["_build"]
author = "Michael Gendy"
autoclass_content = "both"

# Theme
html_theme = 'furo'


html_sidebars = {
    "*": [
        "sidebar/scroll-start.html",
        "sidebar/brand.html",
        "sidebar/search.html",
        "sidebar/navigation.html",
        "sidebar/ethical-ads.html",
        "sidebar/scroll-end.html",
    ]
}






import os
import sys

import django
from django.core import management


def pytest_addoption(parser):
    parser.addoption('--no-pkgroot', action='store_true', default=False,
                     help='Remove package root directory from sys.path, ensuring that '
                          'rest_framework is imported from the installed site-packages. '
                          'Used for testing the distribution.')
    parser.addoption('--staticfiles', action='store_true', default=False,
                     help='Run tests with static files collection, using manifest '
                          'staticfiles storage. Used for testing the distribution.')


def pytest_configure(config):
    from django.conf import settings

    # USE_L10N is deprecated, and will be removed in Django 5.0.
    use_l10n = {"USE_L10N": True} if django.VERSION < (4, 0) else {}
    settings.configure(
        DEBUG_PROPAGATE_EXCEPTIONS=True,
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:'
            },
            'secondary': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:'
            }
        },
        SITE_ID=1,
        SECRET_KEY='not very secret in tests',
        USE_I18N=True,
        MIDDLEWARE=(
            'django.middleware.common.CommonMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
        ),
        INSTALLED_APPS=(
            'django.contrib.admin',
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.staticfiles',
            'rest_framework',
            'rest_framework.authtoken',
            'tests',
        ),
        PASSWORD_HASHERS=(
            'django.contrib.auth.hashers.MD5PasswordHasher',
        ),
        **use_l10n,
    )
    django.setup()



import drf_turbo as dt
from drf_turbo.exceptions import ValidationError
import pytest
import uuid
from decimal import  Decimal
import datetime
from django.utils.timezone import utc
from rest_framework.test import  APISimpleTestCase
from django.core.exceptions import ObjectDoesNotExist


NO_DEFAULT = object()


def get_items(mapping_or_list_of_two_tuples):
    # Tests accept either lists of two tuples, or dictionaries.
    if isinstance(mapping_or_list_of_two_tuples, dict):
        # {value: expected}
        return mapping_or_list_of_two_tuples.items()
    # [(value, expected), ...]
    return mapping_or_list_of_two_tuples


class FieldValues:
    """
    Base class for testing valid and invalid input values.
    """
    context = {}
    
    def test_valid_inputs(self):
        """
        Ensure that valid values return the expected validated data.
        """
        for input_value, expected_output in get_items(self.valid_inputs):
            assert self.field.run_validation(input_value,self.context) == expected_output, \
                'input value: {}'.format(repr(input_value))

    def test_invalid_inputs(self):
        """
        Ensure that invalid values raise the expected validation error.
        """
        for input_value, expected_failure in get_items(self.invalid_inputs):
            with pytest.raises(ValidationError) as exc_info:
                self.field.run_validation(input_value,self.context)
            assert exc_info.value.detail == expected_failure, \
                'input value: {}'.format(repr(input_value))

    def test_outputs(self):
        for output_value, expected_output in get_items(self.outputs):
            assert self.field.serialize(output_value,self.context) == expected_output, \
                'output value: {}'.format(repr(output_value))

class TestStrField(FieldValues):
    """
    Valid and invalid values for `CharField`.
    """
    valid_inputs = {
        1: '1',
        'abc': 'abc'
    }
    invalid_inputs = {
        (): ['Not a valid string.'],
        True: ['Not a valid string.'],
        '': ['May not be blank.']
    }
    outputs = {
        1: '1',
        'abc': 'abc'
    }
    field = dt.StrField()

    def test_trim_whitespace_default(self):
        field =dt.StrField()
        assert field.deserialize(' abc ',self.context) == 'abc'

    def test_trim_whitespace_disabled(self):
        field = dt.StrField(trim_whitespace=False)
        assert field.deserialize(' abc ',self.context) == ' abc '

    def test_disallow_blank_with_trim_whitespace(self):
        field = dt.StrField(allow_blank=False, trim_whitespace=True)

        with pytest.raises(ValidationError) as exc_info:
            field.run_validation('   ',self.context)
        assert exc_info.value.detail == ['May not be blank.']

    def test_null_bytes(self):
        field = dt.StrField()

        for value in ('\0', 'foo\0', '\0foo', 'foo\0foo'):
            with pytest.raises(ValidationError) as exc_info:
                field.run_validation(value,self.context)
            assert exc_info.value.detail == [
                'Null characters are not allowed.'
            ]

class TestEmailField(FieldValues):
    """
    Valid and invalid values for `EmailField`.
    """
    valid_inputs = {
        'example@example.com': 'example@example.com',
    }
    invalid_inputs = {
        'examplecom': ['Enter a valid email address.']
    }
    outputs = {}
    field = dt.EmailField()

class TestRegexField(FieldValues):
    """
    Valid and invalid values for `RegexField`.
    """
    valid_inputs = {
        'a9': 'a9',
    }
    invalid_inputs = {
        'A9': ["This value does not match the required pattern."]
    }
    outputs = {}
    field = dt.RegexField(regex='[a-z][0-9]')


class TestURLField(FieldValues):
    """
    Valid and invalid values for `URLField`.
    """
    valid_inputs = {
        'http://example.com': 'http://example.com',
    }
    invalid_inputs = {
        'example.com': ['Enter a valid URL.']
    }
    outputs = {}
    field = dt.URLField()


class TestUUIDField(FieldValues):
    """
    Valid and invalid values for `UUIDField`.
    """
    valid_inputs = {
        '825d7aeb-05a9-45b5-a5b7-05df87923cda': uuid.UUID('825d7aeb-05a9-45b5-a5b7-05df87923cda'),
    }
    invalid_inputs = {
        '825d7aeb-05a9-45b5-a5b7': ["Not a valid UUID."],
    }
    outputs = {
        uuid.UUID('825d7aeb-05a9-45b5-a5b7-05df87923cda'): '825d7aeb-05a9-45b5-a5b7-05df87923cda'
    }
    field = dt.UUIDField()


class TestURLField(FieldValues):
    """
    Valid and invalid values for `URLField`.
    """
    valid_inputs = {
        'http://example.com': 'http://example.com',
    }
    invalid_inputs = {
        'example.com': ['Enter a valid URL.']
    }
    outputs = {}
    field = dt.URLField()
    
    
class TestSlugField(FieldValues):
    """
    Valid and invalid values for `SlugField`.
    """
    valid_inputs = {
        'slug-99': 'slug-99',
    }
    invalid_inputs = {
        'slug 99': ['Not a valid slug.']
    }
    outputs = {}
    field = dt.SlugField()

    def test_allow_unicode_true(self):
        field = dt.SlugField(allow_unicode=True)

        validation_error = False
        try:
            field.run_validation('slug-99-\u0420',self.context)
        except dt.ValidationError:
            validation_error = True

        assert not validation_error

class TestIntField(FieldValues):
    """
    Valid and invalid values for `IntegerField`.
    """
    valid_inputs = {
        '1': 1,
        1: 1,
    }
    invalid_inputs = {
        0.5: ['A valid integer is required.'],
        'abc': ['A valid integer is required.'],
    }
    outputs = {
        '1': 1,
        '0': 0,
    }
    field = dt.IntField()
    
class TestMinMaxIntField(FieldValues):
    """
    Valid and invalid values for `IntegerField` with min and max limits.
    """
    valid_inputs = {
        '1': 1,
        3: 3,
    }
    invalid_inputs = {
        0: ['Must be greater than or equal to 1.'],
        '4': ['Must be less than or equal to 3.'],
    }
    outputs = {}
    field = dt.IntField(min_value=1, max_value=3)



class TestFloatField(FieldValues):
    """
    Valid and invalid values for `FloatField`.
    """
    valid_inputs = {
        '1': 1.0,
        '0': 0.0,
    }
    invalid_inputs = {
        'abc': ["A valid number is required."]
    }
    outputs = {
        '1': 1.0,
        '0': 0.0,
    }
    field = dt.FloatField()
    
    
class TestMinMaxFloatField(FieldValues):
    """
    Valid and invalid values for `FloatField` with min and max limits.
    """
    valid_inputs = {
        '1': 1,
        3: 3,
        1.0: 1.0,
    }
    invalid_inputs = {
        0.9: ['Must be greater than or equal to 1.'],
        '3.1': ['Must be less than or equal to 3.'],
    }
    outputs = {}
    field = dt.FloatField(min_value=1, max_value=3)



class TestDecimalField(FieldValues):
    """
    Valid and invalid values for `DecimalField`.
    """
    valid_inputs = {
        '12.3': Decimal('12.3'),
        '0.1': Decimal('0.1'),
        10: Decimal('10'),

    }
    invalid_inputs = (
        ('', ["A valid number is required."]),
        (' ', ["A valid number is required."]),
        ('abc', ["A valid number is required."]),
        (Decimal('Nan'), ["A valid number is required."]),
    )
    outputs = {
        '1': '1.0',
        '0': '0.0',
        1: '1.0',
        0: '0.0',
        Decimal('1.09'): '1.1',
        Decimal('0.04'): '0.0'
    }
    field = dt.DecimalField(max_digits=3, decimal_places=1)


class TestAllowNullDecimalField(FieldValues):
    valid_inputs = {
        '': None,
        ' ': None,
    }
    invalid_inputs = {}
    outputs = {
        None: '',
    }
    field = dt.DecimalField(max_digits=3, decimal_places=1, allow_null=True)

class TestAllowNullNoStringCoercionDecimalField(FieldValues):
    valid_inputs = {
        '': None,
        ' ': None,
    }
    invalid_inputs = {}
    outputs = {
        None: None,
    }
    field = dt.DecimalField(max_digits=3, decimal_places=1, allow_null=True, coerce_to_string=False)


class TestMinMaxDecimalField(FieldValues):
    """
    Valid and invalid values for `DecimalField` with min and max limits.
    """
    valid_inputs = {
        '10.0': Decimal('10.0'),
        '20.0': Decimal('20.0'),
    }
    invalid_inputs = {
        '9.9': ['Must be greater than or equal to 10.'],
        '20.1': ['Must be less than or equal to 20.'],
    }
    outputs = {}
    field = dt.DecimalField(
        max_digits=3, decimal_places=1,
        min_value=10, max_value=20
    )
    
    
class TestNoDecimalPlaces(FieldValues):
    valid_inputs = {
        '0.12345': Decimal('0.12345'),
    }
    invalid_inputs = {
        '0.1234567': ['Ensure that there are no more than 6 digits in total.']
    }
    outputs = {
        '1.2345': '1.2345',
        '0': '0',
        '1.1': '1.1',
    }
    field = dt.DecimalField(max_digits=6, decimal_places=None)


class TestNoMaxDigitsDecimalField(FieldValues):
    field = dt.DecimalField(
        max_value=100, min_value=0,
        decimal_places=2, max_digits=None
    )
    valid_inputs = {
        '10': Decimal('10.00')
    }
    invalid_inputs = {}
    outputs = {}

class TestDateField(FieldValues):
    """
    Valid and invalid values for `DateField`.
    """
    valid_inputs = {
        '2001-01-01': datetime.date(2001, 1, 1),
        datetime.date(2001, 1, 1): datetime.date(2001, 1, 1),
    }
    invalid_inputs = {
        'abc': ['Not a valid date.'],
        '2001-99-99': ['Not a valid date.'],
        '2001': ['Not a valid date.'],
        datetime.datetime(2001, 1, 1, 12, 00): ['Expected a date but got a datetime.'],
    }
    outputs = {
        datetime.date(2001, 1, 1): '2001-01-01',
        '2001-01-01': '2001-01-01',
        str('2016-01-10'): '2016-01-10',
        None: None,
        '': None,
    }
    field = dt.DateField()
    
class TestDateTimeField(FieldValues):
    """
    Valid and invalid values for `DateTimeField`.
    """
    valid_inputs = {
        '2001-01-01 13:00': datetime.datetime(2001, 1, 1, 13, 00, tzinfo=utc),
        '2001-01-01T13:00': datetime.datetime(2001, 1, 1, 13, 00, tzinfo=utc),
        '2001-01-01T13:00Z': datetime.datetime(2001, 1, 1, 13, 00, tzinfo=utc),
        datetime.datetime(2001, 1, 1, 13, 00): datetime.datetime(2001, 1, 1, 13, 00, tzinfo=utc),
        datetime.datetime(2001, 1, 1, 13, 00, tzinfo=utc): datetime.datetime(2001, 1, 1, 13, 00, tzinfo=utc),
    }
    invalid_inputs = {
        'abc': ['Not a valid datetime.'],
        '2001-99-99T99:00': ['Not a valid datetime.'],
        '2018-08-16 22:00-24:00': ['Not a valid datetime.'],
        datetime.date(2001, 1, 1): ['Expected a datetime but got a date.'],
        '9999-12-31T21:59:59.99990-03:00': ['Datetime value out of range.'],
    }
    outputs = {
        datetime.datetime(2001, 1, 1, 13, 00): '2001-01-01T13:00:00Z',
        datetime.datetime(2001, 1, 1, 13, 00, tzinfo=utc): '2001-01-01T13:00:00Z',
        '2001-01-01T00:00:00': '2001-01-01T00:00:00',
        str('2016-01-10T00:00:00'): '2016-01-10T00:00:00',
        None: None,
        '': None,
    }
    field = dt.DateTimeField(default_timezone=utc)
    
class TestCustomInputFormatDateTimeField(FieldValues):
    """
    Valid and invalid values for `DateTimeField` with a custom input format.
    """
    valid_inputs = {
        '1:35pm, 1 Jan 2001': datetime.datetime(2001, 1, 1, 13, 35, tzinfo=utc),
    }
    invalid_inputs = {
        '2001-01-01T20:50': ['Not a valid datetime.']
    }
    outputs = {}
    field = dt.DateTimeField(default_timezone=utc, input_formats=['%I:%M%p, %d %b %Y'])


class TestCustomOutputFormatDateTimeField(FieldValues):
    """
    Values for `DateTimeField` with a custom output format.
    """
    valid_inputs = {}
    invalid_inputs = {}
    outputs = {
        datetime.datetime(2001, 1, 1, 13, 00): '01:00PM, 01 Jan 2001',
    }
    field = dt.DateTimeField(format='%I:%M%p, %d %b %Y')


class TestNoOutputFormatDateTimeField(FieldValues):
    """
    Values for `DateTimeField` with no output format.
    """
    valid_inputs = {}
    invalid_inputs = {}
    outputs = {
        datetime.datetime(2001, 1, 1, 13, 00): datetime.datetime(2001, 1, 1, 13, 00),
    }
    field = dt.DateTimeField(format=None)


class TestCustomOutputFormatDateField(FieldValues):
    """
    Values for `DateField` with a custom output format.
    """
    valid_inputs = {}
    invalid_inputs = {}
    outputs = {
        datetime.date(2001, 1, 1): '01 Jan 2001'
    }
    field = dt.DateField(format='%d %b %Y')


class TestTimeField(FieldValues):
    """
    Valid and invalid values for `TimeField`.
    """
    valid_inputs = {
        '13:00': datetime.time(13, 00),
        datetime.time(13, 00): datetime.time(13, 00),
    }
    invalid_inputs = {
        'abc': ['Not a valid time.'],
        '99:99': ['Not a valid time.'],
    }
    outputs = {
        datetime.time(13, 0): '13:00:00',
        datetime.time(0, 0): '00:00:00',
        '00:00:00': '00:00:00',
        None: None,
        '': None,
    }
    field = dt.TimeField()


class TestCustomInputFormatTimeField(FieldValues):
    """
    Valid and invalid values for `TimeField` with a custom input format.
    """
    valid_inputs = {
        '1:00pm': datetime.time(13, 00),
    }
    invalid_inputs = {
        '13:00': ['Not a valid time.'],
    }
    outputs = {}
    field = dt.TimeField(input_formats=['%I:%M%p'])


class TestCustomOutputFormatTimeField(FieldValues):
    """
    Values for `TimeField` with a custom output format.
    """
    valid_inputs = {}
    invalid_inputs = {}
    outputs = {
        datetime.time(13, 00): '01:00PM'
    }
    field = dt.TimeField(format='%I:%M%p')


class TestNoOutputFormatTimeField(FieldValues):
    """
    Values for `TimeField` with a no output format.
    """
    valid_inputs = {}
    invalid_inputs = {}
    outputs = {
        datetime.time(13, 00): datetime.time(13, 00)
    }
    field = dt.TimeField(format=None)

class TestChoiceField(FieldValues):
    """
    Valid and invalid values for `ChoiceField`.
    """
    valid_inputs = {
        'poor': 'poor',
        'medium': 'medium',
        'good': 'good',
    }
    invalid_inputs = {
        'amazing': ['"amazing" is not a valid choice.']
    }
    outputs = {
        'good': {'value' : 'good', 'display' : 'Good quality'},
         '': '',
        'amazing':{'display': 'amazing', 'value': 'amazing'} 
    }
    field = dt.ChoiceField(
        choices=[
            ('poor', 'Poor quality'),
            ('medium', 'Medium quality'),
            ('good', 'Good quality'),
        ]
    )

    def test_allow_blank(self):
        """
        If `allow_blank=True` then '' is a valid input.
        """
        field = dt.ChoiceField(
            allow_blank=True,
            choices=[
                ('poor', 'Poor quality'),
                ('medium', 'Medium quality'),
                ('good', 'Good quality'),
            ]
        )
        output = field.run_validation('',self.context)
        assert output == ''



class TestChoiceFieldWithType(FieldValues):
    """
    Valid and invalid values for a `Choice` field that uses an integer type,
    instead of a char type.
    """
    valid_inputs = {
        '1': 1,
        3: 3,
    }
    invalid_inputs = {
        5: ['"5" is not a valid choice.'],
        'abc': ['"abc" is not a valid choice.']
    }
    outputs = {
        '1': {'display': 'Poor quality', 'value': 1},
        1: {'display': 'Poor quality', 'value': 1},
    }
    field = dt.ChoiceField(
        choices=[
            (1, 'Poor quality'),
            (2, 'Medium quality'),
            (3, 'Good quality'),
        ]
    )

class TestChoiceFieldWithListChoices(FieldValues):
    """
    Valid and invalid values for a `Choice` field that uses a flat list for the
    choices, rather than a list of pairs of (`value`, `description`).
    """
    valid_inputs = {
        'poor': 'poor',
        'medium': 'medium',
        'good': 'good',
    }
    invalid_inputs = {
        'awful': ['"awful" is not a valid choice.']
    }
    outputs = {
        'good': {'display':'good','value' : 'good'}
    }
    field = dt.ChoiceField(choices=('poor', 'medium', 'good'))

class TestMultipleChoiceField(FieldValues):
    """
    Valid and invalid values for `MultipleChoiceField`.
    """
    valid_inputs = {
        (): set(),
        ('aircon',): {'aircon'},
        ('aircon', 'manual'): {'aircon', 'manual'},
    }
    invalid_inputs = {
        'abc': ['Expected a list of items but got type "str".'],
        ('aircon', 'incorrect'): ['"incorrect" is not a valid choice.']
    }
    outputs = [
        (['aircon', 'manual', 'incorrect'], {'aircon', 'manual', 'incorrect'})
    ]
    field = dt.MultipleChoiceField(
        choices=[
            ('aircon', 'AirCon'),
            ('manual', 'Manual drive'),
            ('diesel', 'Diesel'),
        ]
    )

class TestEmptyMultipleChoiceField(FieldValues):
    """
    Invalid values for `MultipleChoiceField(allow_empty=False)`.
    """
    valid_inputs = {
    }
    invalid_inputs = (
        ([], ['This selection may not be empty.']),
    )
    outputs = [
    ]
    field = dt.MultipleChoiceField(
        choices=[
            ('consistency', 'Consistency'),
            ('availability', 'Availability'),
            ('partition', 'Partition tolerance'),
        ],
        allow_empty=False
    )



class TestBooleanField(FieldValues):
    """
    Valid and invalid values for `BooleanField`.
    """
    valid_inputs = {
        'true': True,
        'false': False,
        '1': True,
        '0': False,
        1: True,
        0: False,
        True: True,
        False: False,
    }
    invalid_inputs = {
        'foo': ['Not a valid boolean.'],
        None: ['This field may not be null.']
    }
    outputs = {
        'true': True,
        'false': False,
        '1': True,
        '0': False,
        1: True,
        0: False,
        True: True,
        False: False,
        'other': True
    }
    field = dt.BoolField()

    def test_disallow_unhashable_collection_types(self):
        inputs = (
            [],
            {},
        )
        field = self.field
        for input_value in inputs:
            with pytest.raises(ValidationError) as exc_info:
                field.run_validation(input_value,self.context)
            expected = ['Not a valid boolean.']
            assert exc_info.value.detail == expected


class TestNullBooleanField(TestBooleanField):
    """
    Valid and invalid values for `NullBooleanField`.
    """
    valid_inputs = {
        'true': True,
        'false': False,
        'null': None,
        True: True,
        False: False,
        None: None
    }
    invalid_inputs = {
        'foo': ['Not a valid boolean.'],
    }
    outputs = {
        'true': True,
        'false': False,
        'null': None,
        True: True,
        False: False,
        'other': True
    }
    field = dt.BoolField(allow_null=True)


class TestListField(FieldValues):
    """
    Values for `ListField` with IntegerField as child.
    """
    valid_inputs = [
        ([1, 2, 3], [1, 2, 3]),
        (['1', '2', '3'], [1, 2, 3]),
        ([], [])
    ]
    invalid_inputs = [
        ('not a list', ['Expected a list of items but got type "str".']),
        ([1, 2, 'error', 'error'], {2: ['A valid integer is required.'], 3: ['A valid integer is required.']}),
        ({'one': 'two'}, ['Expected a list of items but got type "dict".'])
    ]
    outputs = [
        ([1, 2, 3], [1, 2, 3]),
        (['1', '2', '3'], [1, 2, 3])
    ]
    field = dt.ArrayField(child=dt.IntField())

    def test_collection_types_are_invalid_input(self):
        field = dt.ArrayField(child=dt.StrField())
        input_value = ({'one': 'two'})

        with pytest.raises(ValidationError) as exc_info:
            field.deserialize(input_value,self.context)
        assert exc_info.value.detail == ['Expected a list of items but got type "dict".']

class TestNestedArrayField(FieldValues):
    """
    Values for nested `ArrayField` with IntegerField as child.
    """
    valid_inputs = [
        ([[1, 2], [3]], [[1, 2], [3]]),
        ([[]], [[]])
    ]
    invalid_inputs = [
        (['not a list'], {0: ['Expected a list of items but got type "str".']}),
        ([[1, 2, 'error'], ['error']], {0: {2: ['A valid integer is required.']}, 1: {0: ['A valid integer is required.']}}),
        ([{'one': 'two'}], {0: ['Expected a list of items but got type "dict".']})
    ]
    outputs = [
        ([[1, 2], [3]], [[1, 2], [3]]),
    ]
    field = dt.ArrayField(child=dt.ArrayField(child=dt.IntField()))


class TestEmptyArrayField(FieldValues):
    """
    Values for `ArrayField` with allow_empty=False flag.
    """
    valid_inputs = {}
    invalid_inputs = [
        ([], ['This list may not be empty.'])
    ]
    outputs = {}
    field = dt.ArrayField(child=dt.IntField(), allow_empty=False)


class TestArrayFieldLengthLimit(FieldValues):
    valid_inputs = ()
    invalid_inputs = [
        ((0, 1), ['Must have at least 3 items.']),
        ((0, 1, 2, 3, 4, 5), ['Must have no more than 4 items.']),
    ]
    outputs = ()
    field = dt.ArrayField(child=dt.IntField(), min_items=3, max_items=4)


class TestArrayFieldExactLength(FieldValues):
    valid_inputs = ()
    invalid_inputs = [
        ((0, 1), ['Must have 3 items.']),
    ]
    outputs = ()
    field = dt.ArrayField(child=dt.IntField(),exact_items=3)


class TestDictField(FieldValues):
    """
    Values for `DictField` with CharField as child.
    """
    valid_inputs = [
        ({'a': 1, 'b': '2', 3: 3}, {'a': '1', 'b': '2', '3': '3'}),
        ({}, {}),
    ]
    invalid_inputs = [
        ({'a': 1, 'b': None, 'c': None}, {'b': ['This field may not be null.'], 'c': ['This field may not be null.']}),
        ('not a dict', ['Expected a dict of items but got type "str".']),
    ]
    outputs = [
        ({'a': 1, 'b': '2', 3: 3}, {'a': '1', 'b': '2', '3': '3'}),
    ]
    field = dt.DictField(child=dt.StrField())

    def test_allow_null(self):
        """
        If `allow_null=True` then `None` is a valid input.
        """
        field = dt.DictField(allow_null=True)
        output = field.run_validation(None,self.context)
        assert output is None

    def test_allow_empty_disallowed(self):
        """
        If allow_empty is False then an empty dict is not a valid input.
        """
        field = dt.DictField(allow_empty=False)
        with pytest.raises(ValidationError) as exc_info:
            field.run_validation({},self.context)

        assert exc_info.value.detail == ['This dict may not be empty.']
        
class TestJSONField(FieldValues):
    """
    Values for `JSONField`.
    """
    valid_inputs = [
        ({
            'a': 1,
            'b': ['some', 'list', True, 1.23],
            '3': None
        }, {
            'a': 1,
            'b': ['some', 'list', True, 1.23],
            '3': None
        }),
    ]
    invalid_inputs = [
        ({'a': set()}, ['Not a valid JSON.']),
    ]
    outputs = [
        ({
            'a': 1,
            'b': ['some', 'list', True, 1.23],
            '3': 3
        }, {
            'a': 1,
            'b': ['some', 'list', True, 1.23],
            '3': 3
        }),
    ]
    field = dt.JSONField()
    
class TestConstantField(FieldValues):
    """
    Values for `ConstantField`.
    """
    field = dt.ConstantField(constant="abc")
    
    valid_inputs = {
        'abc' : 'abc',
    }
    invalid_inputs = {
        'abcd' : ['Must be "abc".']
    }
    outputs = {
        'abc' : 'abc'
    }
    
    def test_disallow_null_constant(self):
        field = dt.ConstantField(constant=None)
        with pytest.raises(ValidationError) as exc_info:
            field.run_validation({},self.context)

        assert exc_info.value.detail == ['Must be None.']
        
        

class MockFile:
    def __init__(self, name='', size=0, url=''):
        self.name = name
        self.size = size
        self.url = url

    def __eq__(self, other):
        return (
            isinstance(other, MockFile) and
            self.name == other.name and
            self.size == other.size and
            self.url == other.url
        )


class TestFileField(FieldValues):
    """
    Values for `FileField`.
    """
    valid_inputs = [
        (MockFile(name='example', size=10), MockFile(name='example', size=10))
    ]
    invalid_inputs = [
        ('invalid', ['The submitted data was not a file. Check the encoding type on the form.']),
        (MockFile(name='example.txt', size=0), ['The submitted file is empty.']),
        (MockFile(name='', size=10), ['No filename could be determined.']),
        (MockFile(name='x' * 100, size=10), ['Ensure this filename has at most 10 characters (it has 100).'])
    ]
    outputs = [
        (MockFile(name='example.txt', url='/example.txt'), '/example.txt'),
        ('', None)
    ]
    field = dt.FileField(max_length=10)




class TestMethodField:
    def test_method_field(self):
        class ExampleSerializer(dt.Serializer):
            example_field = dt.MethodField()

            def get_example_field(self, obj):
                return 'ran get_example_field(%d)' % obj['example_field']
    
        serializer = ExampleSerializer({'example_field': 123})
        assert serializer.data == {
            'example_field': 'ran get_example_field(123)'
        }

    def test_redundant_method_name(self):
        class ExampleSerializer(dt.Serializer):
            example_field = dt.MethodField('get_example_field')

        field = ExampleSerializer().fields['example_field']
        assert field.method_name == 'get_example_field'


class MockObject:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        for key, val in kwargs.items():
            setattr(self, key, val)

    def __str__(self):
        kwargs_str = ', '.join([
            '%s=%s' % (key, value)
            for key, value in sorted(self._kwargs.items())
        ])
        return '<MockObject %s>' % kwargs_str


class MockQueryset:
    def __init__(self, iterable):
        self.items = iterable

    def __getitem__(self, val):
        return self.items[val]

    def get(self, **lookup):
        for item in self.items:
            if all([
                getattr(item, key, None) == value
                for key, value in lookup.items()
            ]):
                return item
        raise ObjectDoesNotExist()


class BadType:
    """
    When used as a lookup with a `MockQueryset`, these objects
    will raise a `TypeError`, as occurs in Django when making
    queryset lookups with an incorrect type for the lookup value.
    """
    def __eq__(self):
        raise TypeError()


class TestRelatedField(APISimpleTestCase):
    context = {}
    def setUp(self):
        self.queryset = MockQueryset([
            MockObject(pk=1, name='foo'),
            MockObject(pk=2, name='bar'),
            MockObject(pk=3, name='baz')
        ])
        self.instance = self.queryset.items[2]
        self.field = dt.RelatedField(queryset=self.queryset)

    def test_pk_related_lookup_exists(self):
        instance = self.field.deserialize(self.instance.pk,self.context)
        assert instance is self.instance

    def test_pk_related_lookup_does_not_exist(self):
        with pytest.raises(ValidationError) as excinfo:
            self.field.deserialize(4,self.context)
        msg = excinfo.value.detail[0]
        assert msg == 'Invalid pk "4" - object does not exist.'

    def test_pk_related_lookup_invalid_type(self):
        with pytest.raises(ValidationError) as excinfo:
            self.field.deserialize(BadType(),self.context)
        msg = excinfo.value.detail[0]
        assert msg == 'Incorrect type. Expected pk value, received BadType.'

    def test_pk_related_lookup_bool(self):
        with pytest.raises(ValidationError) as excinfo:
            self.field.deserialize(True,self.context)
        msg = excinfo.value.detail[0]
        assert msg == 'Incorrect type. Expected pk value, received bool.'

class TestManyRelatedField(APISimpleTestCase):
    context = {}
    def setUp(self):
        self.queryset = MockQueryset([
            MockObject(pk=1, name='foo'),
            MockObject(pk=2, name='bar'),
            MockObject(pk=3, name='baz')
        ])
        self.child_relation = dt.RelatedField(queryset=self.queryset)
        self.instance = self.queryset.items[2]
        self.field = dt.ManyRelatedField(child_relation=self.child_relation,allow_empty=False)

    def test_serialize(self):
        data = self.field.serialize(self.queryset,self.context)
        assert data == [1,2,3]

    def test_deserialize(self):
        data = self.field.deserialize( [1,2,3],self.context)
        assert data == self.queryset.items

    def test_child_relation_is_list(self):
        with pytest.raises(ValidationError) as excinfo:
            self.field.deserialize(self.instance,self.context)
        msg = excinfo.value.detail[0]
        assert msg == 'Expected a list of items but got type "MockObject".'

    def test_child_relation_is_not_empty(self):
        with pytest.raises(ValidationError) as excinfo:
            self.field.deserialize([],self.context)
        msg = excinfo.value.detail[0]
        assert msg == 'This list may not be empty.'
        
        





#cython: language_level=3, boundscheck=False, wraparound=False, initializedcheck=False, cdivision=True

import collections
from django.core.exceptions import ObjectDoesNotExist


cpdef bint is_iterable_and_not_string(arg):
    return (
        isinstance(arg, collections.Iterable) 
        and not isinstance(arg, str)
    )
    
cpdef bint is_collection(obj):
    """Return True if ``obj`` is a collection type, e.g list, tuple, queryset."""
    return is_iterable_and_not_string(obj) and not isinstance(obj, dict)


cpdef get_error_detail(exc_info):
    """
    Translate django ValidationError to readable errors 
    """
    cdef dict error_dict
    cdef list errors
    cdef str k

    try:
        error_dict = exc_info.error_dict
    except AttributeError:
        return [
            (error.message % error.params) if error.params else error.message
            for error in exc_info.error_list]
    return {
        k: [
            (error.message % error.params) if error.params else error.message
            for error in errors
        ] for k, errors in error_dict.items()
    }

cpdef str force_str(object obj, str encoding='utf-8'):
    if isinstance(obj, str):
        return obj
    elif isinstance(obj, bytes):
        return obj.decode(encoding)
    else:
        return str(obj)

cpdef get_execption_detail(exception):
    if isinstance(exception,(list,tuple)):
        return [get_execption_detail(item) for item in exception]
    
    elif isinstance(exception,dict):
        return {key : get_execption_detail(value) for key,value in exception.items()}

    return force_str(exception)


cpdef object get_attribute(object instance, list attrs):
    cdef str attr
    for attr in attrs :
        try:
            if isinstance(instance, dict):
                instance = instance[attr]
            else:
                instance = getattr(instance, attr)
        except ObjectDoesNotExist :
            return None

        if callable(instance):
            try:
                instance = instance()
            except (AttributeError, KeyError) as exc:
                raise ValueError(
                    "Unable to resolve attribute '%s' on %s: %s" % (
                        attr, instance, exc
                    )
                )
    return instance

    
cpdef dict deepcopy(dict data):
    cdef dict output = data.copy()
    cdef str key
    for key, value in output.items():
        output[key] = deepcopy(value) if isinstance(value, dict) else value        
    return output






from django.http import HttpResponse 
from typing import Optional,Any
from rest_framework import status
from django.core.serializers.json import DjangoJSONEncoder
import json

try:
    import ujson
except ImportError:  
    ujson = None  

try:
    import orjson
except ImportError:
    orjson = None  

class JSONResponse(HttpResponse):
    def __init__(self, data : Any,**kwargs) -> None:
        kwargs.setdefault('content_type', 'application/json')
        data = json.dumps(
            data,
            cls=DjangoJSONEncoder,
            ensure_ascii=False,
            allow_nan=False,
            indent=None,
            separators=(",", ":"),
        ).encode("utf-8")
        super().__init__(content=data, **kwargs)
        

class UJSONResponse(HttpResponse):
    def __init__(self, data : Any,**kwargs) -> None:
        assert ujson is not None, "ujson must be installed to use UJSONResponse"
        kwargs.setdefault('content_type', 'application/json')
        data = ujson.dumps(data, ensure_ascii=False).encode("utf-8")
        super().__init__(content=data, **kwargs)


class ORJSONResponse(HttpResponse):
    def __init__(self, data : Any,**kwargs) -> None:
        assert orjson is not None, "orjson must be installed to use ORJSONResponse"
        kwargs.setdefault('content_type', 'application/json')
        data = orjson.dumps(data)
        super().__init__(content=data, **kwargs)


class SuccessResponse :
    
    def __new__(cls,data,message:Optional[str]=None,status_code:Optional[int]=None,default:Any=JSONResponse) -> Any :
        if not message : 
            message = 'Success'
        if not status_code :
            status_code = status.HTTP_200_OK
        return default(dict([
            ('message', message),
            ('data', data),
            ('error' , False)
    ]),status=status_code)



class ErrorResponse : 

    def __new__(cls,data,message:Optional[str]=None,status_code:Optional[int]=None,default: Any=JSONResponse) -> Any :
        if not message : 
            message = 'Bad request'
        if not status_code :
            status_code = status.HTTP_400_BAD_REQUEST
        return default(dict([
            ('message', message),
            ('data', data),
            ('error' , True )
    ]),status=status_code)


cimport cython

from drf_turbo.utils import is_iterable_and_not_string,get_error_detail,is_collection,get_attribute,force_str
from drf_turbo.exceptions import *
from django.core.exceptions import ValidationError as DjangoValidationError
import ipaddress,copy,json,uuid,decimal,re,datetime
from django.core.exceptions import ObjectDoesNotExist
from django.utils.dateparse import (
    parse_date, parse_datetime, parse_time
)
from rest_framework.settings import api_settings
from rest_framework import (
    ISO_8601
)
from django.conf import settings
from django.utils import timezone
from pytz.exceptions import InvalidTimeError

EMAIL_REGEX = re.compile(
    r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*"
    r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|\\[\001-\011\013\014\016-\177])*'
    r")@(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,63}(?<!-)\.?$",
    re.IGNORECASE,
)

URL_REGEX = re.compile(
    r'^(?:http|ftp)s?://'  # http:// or https://
    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain..
    r'localhost|'  # localhost...
    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
    r'(?::\d+)?'  # optional port
    r'(?:/?|[/?]\S+)$', re.IGNORECASE)
    

cdef object NO_DEFAULT = object()


cdef class SkipField(Exception):
    pass


cdef class Field :
    """
    Basic field from which other fields should extend. It applies no
    formatting by default, and should only be used in cases where
    data does not need to be formatted before being serialized or deserialized.
    On error, the name of the field will be returned.

    :param str attr:  The name of the attribute to get the value from when serializing.
    :param bool call: Whether the value should be called after it is retrieved
        from the object. Useful if an object has a method to be serialized. 
    :param bool required: Raise a `ValidationError` if the field value
        is not supplied during deserialization.      
    :param bool write_only: If `True` skip this field during serialization, otherwise
        its value will be present in the serialized data.      
    :param bool read_only: If `True` skip this field during deserialization, otherwise
        its value will be present in the deserialized object.  
    :param str label: A label to use as the name of the serialized field
        instead of using the attribute name of the field.   
    :param list validators: A list of validators to apply against incoming data during deserialization.     
    :param str field_name: The name of field.  
    :param str root: The root(parent) of field.   
    :param default_value: Default value to be used during serialization and deserialization.
    :param initial: The initial value for the field.
    """

    is_method_field = False
    default_error_messages = {
        'required': 'This field is required.',
        'null': 'This field may not be null.',
        }
    _initial = None

    def __init__(
        self,
        basestring attr = None,
        bint call= False,
        bint required= True,
        bint write_only = False,
        bint read_only = False,
        bint allow_null = False,
        basestring label = None,
        basestring help_text = None,
        dict style = None ,
        object validators = None,
        object default_value  = NO_DEFAULT,
        object initial = NO_DEFAULT,
        basestring field_name = None,
        object root = None ,
        dict error_messages = None,
        
    ): 
        required = False if default_value is not NO_DEFAULT else required
        assert not (read_only and write_only), 'May not set both `read_only` and `write_only`'
        assert not (required and default_value is not NO_DEFAULT), 'May not set both `required` and `default_value`'

        self.attr = attr
        self.call = call
        self.required = required
        self.write_only = write_only
        self.read_only = (read_only or call or
                          (attr is not None and '.' in attr)) # type: ignore
        self.allow_null = allow_null
        self.label = label
        self.default_value = default_value
        self.initial = self._initial if (initial is NO_DEFAULT) else initial
        self.help_text = help_text
        self.style = {} if style is None else style
        self.field_name = field_name
        self.root = root
        if validators is None:
            self.validators = []
        elif callable(validators):
            self.validators = [validators]
        elif is_iterable_and_not_string(validators) :
            self.validators = list(validators)
        else:
            raise ValueError(
                "The 'validators' parameter must be a callable "
                "or a collection of callables."
            )

        messages = {}
        for cls in reversed(self.__class__.__mro__):
            messages.update(getattr(cls, 'default_error_messages', {}))
        messages.update(error_messages or {})
        self.error_messages = messages

    def raise_if_fail(self, key: str, **kwargs) :
        """Helper method to make a `ValidationError` with an error message
        from ``self.error_messages``.
        """
        try:
            msg = self.error_messages[key]
        except KeyError as error:
            raise AssertionError(error)
        if isinstance(msg, (str, bytes)):
            msg = msg.format(**kwargs)
        return ValidationError(msg)

        
    cpdef serialize(self,value, dict context):
        """
        Transform the *outgoing* native value into primitive data

        :param value: The outgoing value.
        :param context: The context for the request.
        """
        return value
    
    
    cpdef deserialize(self,data, dict context):
        """
        Transform the *incoming* primitive data into a native value.
        
        :param data: The incoming data.
        :param context: The context for the request.
        """
        return data
    

    cpdef method_getter(self,field_name, root) :
        """
        Returns a function that fetches an attribute from an object.

        :field_name: The name of the attribute to get.
        :root: The root of the field.
        """
        return None
        
        
    cpdef void bind(self,basestring field_name, object root):
        """
        Update the field name and root for the field instance.

        :field_name: The name of the field.
        :root: The root of the field.
        """
        self.field_name = field_name
        self.root = root
        if self.label is None:
            self.label = field_name.replace('_', ' ').capitalize()

        if self.attr is None:
            self.attr = field_name

        self.attrs = self.attr.split('.') if self.attr else []


    cpdef get_default_value(self):
        """
        Return the default value for this field.
        """
        if self.default_value is NO_DEFAULT or getattr(self.root, 'partial', False):
            raise SkipField()
        if callable(self.default_value):
            return self.default_value()
        return self.default_value

    cpdef get_initial(self):
        """
        Return the initial value for this field.
        """
        if callable(self.initial):
            return self.initial()
        return self.initial

    cpdef get_attribute(self, instance , attr=None):
        """
        Return the value of the field from the provided instance.
        """
        try:
            if attr is None:
                return get_attribute(instance, self.attrs)
            return get_attribute(instance, attr)
        except (KeyError, AttributeError) as exc:
            if self.default_value is not NO_DEFAULT:
                return self.get_default_value()
            if self.allow_null:
                return None
            if not self.required:
                raise SkipField()
            msg = (
                'Got {exc_type} when attempting to get a value for field'.format(
                    exc_type=type(exc).__name__,
                )
            )
            raise type(exc)(msg)

    cpdef tuple validate_empty_values(self, data):
        """
        Validate empty values, and either:
        * Raise `ValidationError`, indicating invalid data.
        * Return (True, data), indicating an empty value that should be
          returned without any further validation being applied.
        * Return (False, data), indicating a non-empty value, that should
          have validation applied as normal.
        """
        if self.read_only:
            return (True, self.get_default_value())

        if data is NO_DEFAULT:
            if getattr(self.root, 'partial', False):
                raise SkipField()
            if self.required:
                raise self.raise_if_fail('required')
            return (True, self.get_default_value())

        if data is None:
            if not self.allow_null:
                raise self.raise_if_fail('null')
            return (True, None)

        return (False, data)

    cpdef run_validation(self,object data,dict context) :
        """
        Validate an input data.
        """
        cdef bint is_empty_value
        (is_empty_value, data) = self.validate_empty_values(data)
        if is_empty_value:
            return data
        value = self.deserialize(data,context)
        self.validate_or_raise(value)
        return value
    
    
    cpdef long validate_or_raise(self,value) except -1 :
        """
        Validate the value and raise a `ValidationError` if validation fails.
        """

        cdef list errors = []
        for validator in self.validators :
            try :
                validator(value)
            except ValidationError as exc:
                if isinstance(exc.detail, dict):
                    raise   
                errors.extend(exc.detail)
            except DjangoValidationError as exc:
                errors.extend(get_error_detail(exc))
        if errors:
            raise ValidationError(errors)
        
        


        
cdef class StrField(Field):
    """"
    A field that validates input as an string.

    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    
    default_error_messages = {
        'blank': 'May not be blank.',
        'invalid': 'Not a valid string.',
        'null' : 'Null characters are not allowed.',
        'min_length' : 'Must have at least {min_length} characters.',
        'max_length': 'Must have no more than {max_length} characters.',

    }    
    _initial = ''
    def __init__(self,**kwargs) :
        self.allow_blank = kwargs.pop('allow_blank', False)
        self.trim_whitespace = kwargs.pop('trim_whitespace', True)
        self.max_length = kwargs.pop('max_length', None)
        self.min_length = kwargs.pop('min_length', None)
        super().__init__(**kwargs)
         
    cpdef serialize(self,value,dict context) :
        return str(value)

    cpdef deserialize(self,data,dict context) :
        if data == '' or (self.trim_whitespace and str(data).strip() == ''):
            if not self.allow_blank:
                raise self.raise_if_fail('blank')
        if isinstance(data, bool) or not isinstance(data, (str, int, float,)):
           raise self.raise_if_fail('invalid')

        if self.min_length is not None:
            if len(data) < self.min_length:
                raise self.raise_if_fail("min_length",min_length=self.min_length)
        if self.max_length is not None:
            if len(data) > self.max_length:
                raise self.raise_if_fail("max_length",max_length=self.max_length)
        if '\x00' in str(data):
            raise self.raise_if_fail('null')

        data = str(data)
        return data.strip() if self.trim_whitespace else data
    
@cython.final
cdef class EmailField(StrField):
    """
    A field that validates input as an E-Mail address.

    :param to_lower: If True, convert the value to lowercase before validating.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        'invalid': 'Enter a valid email address.'
    }
    def __init__(self, **kwargs): 
        self.to_lower = kwargs.pop('to_lower', False)
        super().__init__(**kwargs)
        
    cpdef inline serialize(self,value,dict context):
        if self.to_lower :
            return value.lower()
        return value
    
    cpdef inline deserialize(self,data,dict context):
        match = EMAIL_REGEX.match(data)
        if not match:
            raise self.raise_if_fail("invalid")
        data = str(data)
        if self.to_lower :
            return data.lower()
        return data

@cython.final
cdef class URLField(StrField):
    """
    A field that validates input as an URL.

    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        'invalid': 'Enter a valid URL.'
    }

    cpdef inline deserialize(self,data,dict context):
        match = URL_REGEX.match(data)
        if not match:
            raise self.raise_if_fail("invalid")
        data = str(data)
        return data 

    

@cython.final
cdef class RegexField(StrField):
    """
    A field that validates input against a given regular expression.

    :param regex: The regular expression to use for validation.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        'invalid': 'This value does not match the required pattern.'
    }

    def __init__(self, regex, **kwargs):
        self.regex = regex
        super().__init__(**kwargs)
        
    cpdef inline deserialize(self,data,dict context):
        compiled = re.compile(self.regex,flags=0)
        match = compiled.search(str(data))
        if not match:
            raise self.raise_if_fail("invalid")
        return str(data)


@cython.final
cdef class IPField(StrField):
    """
    A field that validates that input is an IP address.

    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        'invalid': 'Enter a valid IPv4 or IPv6 address.',
    }
        
    cpdef inline deserialize(self,data,dict context):
        try:
           return ipaddress.ip_address(data)
        except (ValueError, TypeError) :
            raise self.raise_if_fail('invalid')
        
@cython.final
cdef class PasswordField(StrField):
    """
    A field that validates input as a password.

    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    def __init__(self,**kwargs):
        kwargs['write_only'] = True
        kwargs['min_length'] = 4
        kwargs['required'] = True
        super().__init__(**kwargs)


@cython.final
cdef class UUIDField(StrField):
    """
    A field that validates input as an UUID.

    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        
        "invalid": "Not a valid UUID."
    }

    cpdef inline deserialize(self,data,dict context):
        if data is None:
            return None
        if isinstance(data, uuid.UUID):
            return data
        try:
            if isinstance(data, bytes) and len(data) == 16:
                return uuid.UUID(bytes=data)
            elif isinstance(data,int):
                return uuid.UUID(int=data)
            elif isinstance(data,str):
                return uuid.UUID(hex=data)
            else:
                return uuid.UUID(data)
        except :
            raise self.raise_if_fail('invalid')

@cython.final
cdef class SlugField(Field):
    """
    Slug field type.

    :param allow_unicode: If True, allow unicode characters in the field.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'invalid': 'Not a valid slug.',
        'invalid_unicode' : 'Nnot a valid unicode slug.'
    }

    def __init__(self, allow_unicode=False, **kwargs):
        self.allow_unicode = allow_unicode
        super().__init__(**kwargs)

    cpdef inline deserialize(self,data,dict context):
        if self.allow_unicode :
            regex = re.compile(r'^[-\w]+\Z', re.UNICODE)
        else:
            regex = re.compile(r'^[-a-zA-Z0-9_]+$')
        compiled = re.compile(regex,flags=0)
        match = compiled.search(str(data))
        if not match and not self.allow_unicode:
            raise self.raise_if_fail("invalid")
        elif not match and self.allow_unicode:
            raise self.raise_if_fail("invalid_unicode")
        return str(data)

@cython.final
cdef class IntField(Field):
    """
    A field that validates input as an integer.

    :param min_value: The minimum value allowed.
    :param max_value : The maximum value allowed.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'invalid': 'A valid integer is required.',
        'min_value': 'Must be greater than or equal to {min_value}.',
        'max_value': 'Must be less than or equal to {max_value}.',
    }
    re_decimal = re.compile(r'\.0*\s*$')  # allow e.g. '1.0' as an int, but not '1.2'
    def __init__(self,**kwargs):
        self.max_value = kwargs.pop('max_value', None)
        self.min_value = kwargs.pop('min_value', None)
        super().__init__(**kwargs)
        
    cpdef inline serialize(self,value,dict context):
        return int(value)

    cpdef inline deserialize(self,data,dict context):
        try:
            data = int(self.re_decimal.sub('', str(data)))
        except (ValueError, TypeError):
            raise self.raise_if_fail('invalid')
        if self.min_value is not None and data < self.min_value:
            raise self.raise_if_fail("min_value",min_value=self.min_value)
        if self.max_value is not None and data > self.max_value:
            raise self.raise_if_fail("max_value",max_value=self.max_value)

        return data

@cython.final
cdef class FloatField(Field):
    """
    A field that validates input as a float.

    :param min_value: The minimum value allowed.
    :param max_value: The maximum value allowed.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        'invalid': 'A valid number is required.',
        'min_value': 'Must be greater than or equal to {min_value}.',
        'max_value': 'Must be less than or equal to {max_value}.',

    }
    
    def __init__(self,**kwargs):
        self.max_value = kwargs.pop('max_value', None)
        self.min_value = kwargs.pop('min_value', None)
        super().__init__(**kwargs)

    cpdef inline serialize(self,value,dict context):
        return float(value)
    
    cpdef inline deserialize(self,data,dict context):
        try:
            data =  float(data)
        except (TypeError, ValueError):
           raise self.raise_if_fail('invalid')

        if self.min_value is not None and data < self.min_value:
            raise self.raise_if_fail("min_value",min_value=self.min_value)
        if self.max_value is not None and data > self.max_value:
            raise self.raise_if_fail("max_value",max_value=self.max_value)

        return data

@cython.final
cdef class DecimalField(Field):
    """
    A field that validates input as a Python Decimal.

    :param max_digits(required): Maximum number of digits.
    :param deciaml_places(required): Number of decimal places.
    :param min_value: The minimum value allowed.
    :param max_value: The maximum value allowed.
    :param coerce_to_string: If True, values will be converted to strings during serialization.
    :param rounding: How to round the value during serialization.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        'invalid': 'A valid number is required.',
        'max_value': 'Must be less than or equal to {max_value}.',
        'min_value': 'Must be greater than or equal to {min_value}.',
        'max_digits': 'Ensure that there are no more than {max_digits} digits in total.',
        'max_decimal_places': 'Ensure that there are no more than {max_decimal_places} decimal places.',
        'max_whole_digits': 'Ensure that there are no more than {max_whole_digits} digits before the decimal point.',
        'max_string_length': 'String value too large.'
    }
    MAX_STRING_LENGTH = 1000  # Guard against malicious string inputs.

    def __init__(self, max_digits, decimal_places, max_value=None, min_value=None,coerce_to_string=None,rounding=None,**kwargs):
        self.max_digits = max_digits
        self.decimal_places = decimal_places
        self.max_value = max_value
        self.min_value = min_value
        if self.max_digits is not None and self.decimal_places is not None:
            self.max_whole_digits = self.max_digits - self.decimal_places
        else:
            self.max_whole_digits = None

        if coerce_to_string is None:
            self.coerce_to_string = api_settings.COERCE_DECIMAL_TO_STRING
        else:
            self.coerce_to_string = coerce_to_string

        super().__init__(**kwargs)
      
        if rounding is not None:
            valid_roundings = [v for k, v in vars(decimal).items() if k.startswith('ROUND_')]
            assert rounding in valid_roundings, (
                'Invalid rounding option %s. Valid values for rounding are: %s' % (rounding, valid_roundings))
        self.rounding = rounding


    cpdef inline serialize(self,value,dict context):
        if value is None:
            if self.coerce_to_string:
                return ''
            else:
                return None

        if not isinstance(value, decimal.Decimal):
            value = decimal.Decimal(str(value).strip())

        quantized = self.quantize(value)
        if not self.coerce_to_string:
            return quantized

        return '{:f}'.format(quantized)

      

    cpdef inline deserialize(self,data,dict context):
        """
        Validate that the input is a decimal number and return a Decimal
        instance.
        """

        data = force_str(data).strip()    

        if data == '' and self.allow_null:
            return None

        if len(data) > self.MAX_STRING_LENGTH:
            raise self.raise_if_fail('max_string_length')

        try:
            value = decimal.Decimal(data)
        except decimal.DecimalException:
            raise self.raise_if_fail('invalid')

        if value.is_nan():
            raise self.raise_if_fail('invalid')

        if value in (decimal.Decimal('Inf'), decimal.Decimal('-Inf')):
            raise self.raise_if_fail('invalid')

        if self.min_value is not None and value < self.min_value:
            raise self.raise_if_fail("min_value",min_value=self.min_value)

        if self.max_value is not None and value > self.max_value:
            raise self.raise_if_fail("max_value",max_value=self.max_value)

        return self.quantize(self.validate_precision(value))

    cdef inline validate_precision(self, value):
        """
        Ensure that there are no more than max_digits in the number, and no
        more than decimal_places digits after the decimal point.
        Override this method to disable the precision validation for input
        values or to enhance it in any way you need to.
        """
        sign, digittuple, exponent = value.as_tuple()

        if exponent >= 0:
            # 1234500.0
            total_digits = len(digittuple) + exponent
            whole_digits = total_digits
            decimal_places = 0
        elif len(digittuple) > abs(exponent):
            # 123.45
            total_digits = len(digittuple)
            whole_digits = total_digits - abs(exponent)
            decimal_places = abs(exponent)
        else:
            # 0.001234
            total_digits = abs(exponent)
            whole_digits = 0
            decimal_places = total_digits

        if self.max_digits is not None and total_digits > self.max_digits:
           raise self.raise_if_fail('max_digits', max_digits=self.max_digits)
        if self.decimal_places is not None and decimal_places > self.decimal_places:
            raise self.raise_if_fail('max_decimal_places', max_decimal_places=self.decimal_places)
        if self.max_whole_digits is not None and whole_digits > self.max_whole_digits:
            raise self.raise_if_fail('max_whole_digits', max_whole_digits=self.max_whole_digits)

        return value

    cdef inline quantize(self, value):
        """
        Quantize the decimal value to the configured precision.
        """
        if self.decimal_places is None:
            return value

        context = decimal.getcontext().copy()
        if self.max_digits is not None:
            context.prec = self.max_digits
        return value.quantize(
            decimal.Decimal('.1') ** self.decimal_places,
            rounding=self.rounding,
            context=context
        )


@cython.final
cdef class BoolField(Field):
    """
    Boolean field type.

    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'invalid': 'Not a valid boolean.'
    }
    _initial = False
    coerce_values = {
        "true": True,
        "True": True,
        "TRUE": True,
        't' : True,
        "T" : True,
        "on": True,
        "1": True,
        True : True,
        1: True,
        "off": False,
        "f" : False,
        "F" : False,
        "false": False,
        "False":False,
        "FALSE" : False,
        "0": False,
        "": False,
        False : False,
        0: False,
    }
    coerce_null_values = {"", "null","Null","NULL","none","None","NONE"}
        
    cpdef inline serialize(self,value,dict context):
        try:
            if self.allow_null and value in self.coerce_null_values:
                return None
            value = self.coerce_values[value]
        except (KeyError,TypeError):
            pass
        return bool(value)
    
    cpdef inline deserialize(self,data,dict context):
        try:
            if self.allow_null and data in self.coerce_null_values:
                return None
            data = self.coerce_values[data]
        except (KeyError,TypeError):
            raise self.raise_if_fail('invalid')
        return data
    
cdef class ChoiceField(Field):
    """
    Choice field type.

    :param choices(required): A list of valid choices.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'invalid_choice': '"{input}" is not a valid choice.'
    }

    def __init__(self, choices, **kwargs):
        
        pairs = [
            isinstance(item, (list, tuple)) and len(item) == 2
            for item in choices
        ]
        if all(pairs):
            self.choices = dict([(key, display_value) for key, display_value in choices])
        else:
            self.choices = dict([(item, item) for item in choices])

        self.choice_strings_to_values = dict([
            (str(key), key) for key in self.choices.keys()
        ])
        
        self.choice_strings_to_display = dict([
            (str(key), value) for key,value in self.choices.items()
        ])


        self.allow_blank = kwargs.pop('allow_blank', False)
        super().__init__(**kwargs)

        
        
    cpdef serialize(self,value,dict context):
        if value in ('', None):
            return value
        
        return {
            'value': self.choice_strings_to_values.get(str(value), value),
            'display': self.choice_strings_to_display.get(str(value), value),
        }

        
    cpdef deserialize(self,data,dict context) :
        if data == '' and self.allow_blank:
            return ''
        try:
            return self.choice_strings_to_values[str(data)]
        except:
            raise self.raise_if_fail('invalid_choice', input=data)


@cython.final
cdef class MultipleChoiceField(ChoiceField):
    """
    Multiple choice field type.

    :param allow_empty: If True, allow the user to leave the field blank.
    :param kwargs: The same keyword arguments that :class:`Field` and class:`ChoiceField` receives.
    """
    default_error_messages = {
        'not_a_list': 'Expected a list of items but got type "{input_type}".',
        'empty': 'This selection may not be empty.',
        'invalid_choice': '"{input}" is not a valid choice.'
    }

    def __init__(self, **kwargs):
        self.allow_empty = kwargs.pop('allow_empty', True)
        super().__init__(**kwargs)


    cpdef inline serialize(self,value,dict context):
        return {
            self.choice_strings_to_values.get(str(item), item) for item in value
        }

    cpdef inline deserialize(self,data,dict context) :
        if isinstance(data, str) or not hasattr(data, '__iter__'):
            raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
        if not self.allow_empty and len(data) == 0:
            raise self.raise_if_fail('empty')

        new_data = set()
        for item in data :
            if item == '' and self.allow_blank:
                return ''
            try:
                new_data.add(self.choice_strings_to_values[str(item)])
            except:
                raise self.raise_if_fail('invalid_choice', input=item)
        return new_data




@cython.final
cdef class DateTimeField(Field):
    """
    A field that (de)serializes to a :class:`datetime.datetime` object. 

    :param format: The format to use when serializing/deserializing.
    :param input_formats: A list of formats to check for when deserializing input.
    :param default_timezone: The timezone to use when creating datetime instances.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'invalid': 'Not a valid datetime.',
        'date': 'Expected a datetime but got a date.',
        'make_aware':'Invalid datetime for the timezone "{timezone}".',
        'overflow': 'Datetime value out of range.'
    }       
    datetime_parser = datetime.datetime.strptime 

    def __init__(self, format=NO_DEFAULT,input_formats=None,default_timezone=None,**kwargs):
        if format is NO_DEFAULT:
            self.format = api_settings.DATETIME_FORMAT
        else:
            self.format = format
        if input_formats is None:
            self.input_formats = api_settings.DATETIME_INPUT_FORMATS
        else:
            self.input_formats = input_formats

        if default_timezone is None:
            self.timezone = self.get_default_timezone()
        else:
            self.timezone = default_timezone

        super().__init__(**kwargs)

    cpdef inline get_default_timezone(self):
        return timezone.get_current_timezone() if settings.USE_TZ else None


    cpdef inline enforce_timezone(self, value):
        """
        When `self.default_timezone` is `None`, always return naive datetimes.
        When `self.default_timezone` is not `None`, always return aware datetimes.
        """
        if self.timezone is not None:
            if timezone.is_aware(value):
                try:
                    return value.astimezone(self.timezone)
                except OverflowError:
                   raise self.raise_if_fail('overflow')
            try:
                return timezone.make_aware(value, self.timezone)
            except InvalidTimeError:
                raise self.raise_if_fail('make_aware', timezone=self.timezone)
        elif (self.timezone is None) and timezone.is_aware(value):
            return timezone.make_naive(value, timezone.utc)
        return value


    cpdef inline serialize(self,value,dict context):
        if not value:
            return None

        if self.format is None or isinstance(value, str):
            return value

        value = self.enforce_timezone(value)

        if self.format.lower() == ISO_8601:
            value = value.isoformat()
            if value.endswith('+00:00'):
                value = value[:-6] + 'Z'
            return value
        return value.strftime(self.format)

    
    cpdef inline deserialize(self,data,dict context) :


        if isinstance(data, datetime.date) and not isinstance(data, datetime.datetime):
            raise self.raise_if_fail('date')

        if isinstance(data, datetime.datetime):
            return self.enforce_timezone(data)

        for input_format in self.input_formats:
            if input_format.lower() == ISO_8601:
                try:
                    parsed = parse_datetime(data)
                    if parsed is not None:
                        return self.enforce_timezone(parsed)
                except (ValueError, TypeError):
                    pass
            else:
                try:
                    parsed = self.datetime_parser(data, input_format)
                    return self.enforce_timezone(parsed)
                except (ValueError, TypeError):
                    pass

        raise self.raise_if_fail('invalid')
            
@cython.final
cdef class DateField(Field):
    """
    A field that (de)serializes to a :class:`datetime.date` object.
    
    :param format: Either ``"%Y-%m-%d"`` or ``"%m/%d/%Y"``, or a custom
        string of the format passed to ``strftime``.
    :param input_formats: A list of strings, or a custom list of input formats
        to be used to parse the input date string.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        'invalid': 'Not a valid date.',
        'datetime': 'Expected a date but got a datetime.',
    }

    def __init__(self, format=None,input_formats=None,**kwargs):
        if format is None:
            self.format = api_settings.DATE_FORMAT
        else:
            self.format = format

        if input_formats is None:
            self.input_formats = api_settings.DATE_INPUT_FORMATS
        else:
            self.input_formats = input_formats

        super().__init__(**kwargs)


    cpdef inline serialize(self, value,dict context):
        if not value:
            return None
        if isinstance(value, str):
            return value
        assert not isinstance(value, datetime.datetime), (
            'Expected a `date`, but got a `datetime`. Refusing to coerce, '
            'as this may mean losing timezone information. Use a custom '
            'read-only field and deal with timezone issues explicitly.'
        )

        if self.format.lower() == ISO_8601:
            return value.isoformat()

        return value.strftime(self.format)


    cpdef inline deserialize(self, data,dict context) :

        if isinstance(data, datetime.datetime):
            raise self.raise_if_fail('datetime')

        if isinstance(data, datetime.date):
            return data

        for input_format in self.input_formats:
            if input_format.lower() == ISO_8601:
                try:
                    parsed = parse_date(data)
                except (ValueError, TypeError):
                    pass
                else:
                    if parsed is not None:
                        return parsed
            else:
                try:
                    parsed = self.datetime_parser(data, input_format)
                except (ValueError, TypeError):
                    pass
                else:
                    return parsed.date()

        raise self.raise_if_fail('invalid')


@cython.final
cdef class TimeField(Field):
    """
    A field that (de)serializes to a :class:`datetime.time` object.

    :param format: Either "time" or "datetime" for serialization to display
        time in either 24 hour or 12 hour+minute+second format.
    :param input_formats: Optional list of formats to also be accepted.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'invalid': 'Not a valid time.',
    }
    datetime_parser = datetime.datetime.strptime

    def __init__(self, format=NO_DEFAULT,input_formats=None, **kwargs):
        if format is NO_DEFAULT:
            self.format = api_settings.DATETIME_FORMAT
        else:
            self.format = format
        if input_formats is None:
            self.input_formats = api_settings.TIME_INPUT_FORMATS
        else:
            self.input_formats = input_formats

        super().__init__(**kwargs)


    cpdef inline serialize(self, value,dict context):
        if value in (None, ''):
            return None


        if self.format is None or isinstance(value, str):
            return value

        assert not isinstance(value, datetime.datetime), (
            'Expected a `time`, but got a `datetime`. Refusing to coerce, '
            'as this may mean losing timezone information. Use a custom '
            'read-only field and deal with timezone issues explicitly.'
        )

        if self.format.lower() == ISO_8601:
            return value.isoformat()
        return value.strftime(self.format)



    cpdef inline deserialize(self, data, dict context):

        if isinstance(data, datetime.time):
            return data

        for input_format in self.input_formats:
            if input_format.lower() == ISO_8601:
                try:
                    parsed = parse_time(data)
                except (ValueError, TypeError):
                    pass
                else:
                    if parsed is not None:
                        return parsed
            else:
                try:
                    parsed = self.datetime_parser(data, input_format)
                except (ValueError, TypeError):
                    pass
                else:
                    return parsed.time()

        raise self.raise_if_fail('invalid')



@cython.final
cdef class FileField(Field):
    """
    A file field.    

    :param max_length: The maximum file size.
    :param alow_empty_file: Whether to allow uploading empty files.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'required': 'No file was submitted.',
        'invalid': 'The submitted data was not a file. Check the encoding type on the form.',
        'no_name': 'No filename could be determined.',
        'empty': 'The submitted file is empty.',
        'max_length': 'Ensure this filename has at most {max_length} characters (it has {length}).',
    }

    def __init__(self,**kwargs):
        self.max_length = kwargs.pop('max_length', None)
        self.allow_empty_file = kwargs.pop('allow_empty_file', False)
        super().__init__(**kwargs)

        
    cpdef inline serialize(self,value,dict context):
        if not value :
            return None
        try:
            request = context.get('request', None)
            return request.build_absolute_uri(value.url)
        except:
            return value.url
        
        
    cpdef inline deserialize(self,value,dict context):
        try:
            file_name = value.name
            file_size = value.size
        except AttributeError:
            raise self.raise_if_fail('invalid')
        if not file_name:
            raise self.raise_if_fail('no_name')
        if not self.allow_empty_file and not file_size:
            raise self.raise_if_fail('empty')
        if self.max_length and len(file_name) > self.max_length:
            raise self.raise_if_fail('max_length',max_length=self.max_length, length=len(file_name))

        return value
    
@cython.final
cdef class ArrayField(Field):
    """
    An Array Field.

    :param child: The field to validate the array elements.
    :param allow_empty: Whether the array can be empty.
    :param max_items: The maximum number of items allowed in the array.
    :param min_items: The minimum number of items required in the array.
    :param exac_items: The exact number of items required in the array.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """

    default_error_messages = {
        'not_a_list': 'Expected a list of items but got type "{input_type}".',
        'empty': 'This list may not be empty.',
        'exact_items': 'Must have {exact_items} items.',
        'min_items' : 'Must have at least {min_items} items.',
        'max_items' : 'Must have no more than {max_items} items.',

    }
    _initial = []

    def __init__(self,child=None,**kwargs):
        self.child = kwargs.pop('child', copy.deepcopy(child))
        self.allow_empty = kwargs.pop('allow_empty', True)
        self.max_items = kwargs.pop('max_items', None)
        self.min_items = kwargs.pop('min_items', None)
        self.exact_items =  kwargs.pop('exact_items', None)
        super().__init__(**kwargs)
        if self.child is not None:
            self.child.bind(field_name='', root=self)
        if self.exact_items is not None:
            self.min_items = self.exact_items
            self.max_items = self.exact_items

                    
    cpdef inline serialize(self,data ,dict context):
        """
        List of object instances -> List of dicts of primitive datatypes.
        """
        return [ self.child.serialize(item,context) if item is not None else None for item in data]
    
    cpdef inline deserialize(self,data,dict context):
        """
        List of dicts of native values <- List of dicts of primitive datatypes.
        """
        if isinstance(data, (str, dict)) or not hasattr(data, '__iter__'):
            raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
        if not self.allow_empty and len(data) == 0:
            raise self.raise_if_fail('empty')
        if (
            self.min_items is not None
            and self.min_items == self.max_items
            and len(data) != self.min_items
        ):
            raise self.raise_if_fail("exact_items",exact_items=self.exact_items)
        if self.min_items is not None and len(data) < self.min_items:
            raise self.raise_if_fail("min_items",min_items=self.min_items)
        elif self.max_items is not None and len(data) > self.max_items:
            raise self.raise_if_fail("max_items",max_items=self.max_items)

        return self.run_child_validation(data,context)

    cpdef inline run_child_validation(self,data,dict context):
        cdef list result = []
        cdef dict errors = {}
        cdef int idx = 0
        for item in data :
            try:
                result.append(self.child.run_validation(item,context))
            except ValidationError as e:
                errors[idx] = e.detail
            idx +=1

        if not errors:
            return result
        raise ValidationError(errors)
    
@cython.final
cdef class DictField(Field):
    """A Dict Field.

    :param child: The field to validate the dict values.
    :param allow_empty: Whether the dict can be empty.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'not_a_dict': 'Expected a dict of items but got type "{input_type}".',
        'empty': 'This dict may not be empty.',
    }
    _initial = {}

    def __init__(self,child=None,**kwargs):
        self.child = kwargs.pop('child', copy.deepcopy(child))
        self.allow_empty = kwargs.pop('allow_empty', True)
        super().__init__(**kwargs)
        if self.child is not None:
            self.child.bind(field_name='', root=self)

    cpdef inline serialize(self, data,dict context):
        """
        List of object instances -> List of dicts of primitive datatypes.
        """
        return {
            str(key) : self.child.serialize(value,context) if value is not None else None for key,value in data.items()
        }
    
    cpdef inline deserialize(self,data,dict context):
        """
        List of dicts of native values <- List of dicts of primitive datatypes.
        """
        if not isinstance(data,dict):
            raise self.raise_if_fail('not_a_dict', input_type=type(data).__name__)
        if not self.allow_empty and len(data) == 0:
            raise self.raise_if_fail('empty')

        return self.run_child_validation(data,context)

    cpdef inline run_child_validation(self,dict data,dict context):
        cdef dict result = {}
        cdef dict errors = {}
        for key,value in data.items():
            try:
                result[str(key)] = self.child.run_validation(value,context)
            except ValidationError as e:
                errors[key] = e.detail

        if not errors:
            return result
        raise ValidationError(errors)

@cython.final
cdef class JSONField(Field):
    """A JSON Field.

    :param binary: Whether to load/dump JSON as binary data.
    :param encoder: The JSON encoder class to use.
    :param decoder: The JSON decoder class to use.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'invalid': 'Not a valid JSON.'
    }

    def __init__(self, **kwargs):
        self.binary = kwargs.pop('binary', False)
        self.encoder = kwargs.pop('encoder', None)
        self.decoder = kwargs.pop('decoder', None)
        super().__init__(**kwargs)

    cpdef inline serialize(self,value,dict context):
        if self.binary:
            value = json.dumps(value,cls=self.encoder)
            value = value.encode()
        return value

    cpdef inline deserialize(self,data,dict context):
        try:
            if self.binary:
                if isinstance(data, bytes):
                    data = data.decode()
                return json.loads(data,cls=self.decoder)
            else:
                json.dumps(data,cls=self.encoder)
        except (TypeError, ValueError) as e:
            raise self.raise_if_fail('invalid')
        return data


@cython.final
cdef class RelatedField(Field):
    """A Related Field.

    :param queryset: The queryset to use for getting the instance from the given value.
    """
    default_error_messages = {
        'does_not_exist': 'Invalid pk "{pk_value}" - object does not exist.',
        'incorrect_type': 'Incorrect type. Expected pk value, received {data_type}.',

    }
    def __init__(self,**kwargs):
        self.queryset = kwargs.pop('queryset', None)
        super().__init__(**kwargs)
        
    cpdef inline deserialize(self,data,dict context):
        try:
            if isinstance(data, bool):
                raise TypeError
            data = self.queryset.get(pk=data)
        except ObjectDoesNotExist:
            raise self.raise_if_fail('does_not_exist', pk_value=data)
        except (TypeError, ValueError): 
            raise self.raise_if_fail('incorrect_type', data_type=type(data).__name__)
        return data

    
@cython.final
cdef class ManyRelatedField(Field):
    """
    A field used to represent a to-many relationship.

    :param child_relation: The model field that is the reverse of the relation.
    :param allow_empty: Whether the list can be empty.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'not_a_list': 'Expected a list of items but got type "{input_type}".',
        'empty': 'This list may not be empty.'
    }
    def __init__(self,**kwargs):
        self.child_relation = kwargs.pop('child_relation', None)
        self.allow_empty = kwargs.pop('allow_empty', True)
        super().__init__(**kwargs)
        
    cpdef inline serialize(self,value,dict context):
        
        value = value.all() if hasattr(value, 'all') else value
        return [
            item.pk for item in value
        ]            

    cpdef inline deserialize(self, data,dict context):
        if isinstance(data, str) or not hasattr(data, '__iter__'):
            raise self.raise_if_fail('not_a_list', input_type=type(data).__name__)
        if not self.allow_empty and len(data) == 0:
            raise self.raise_if_fail('empty')
        return [
            self.child_relation.deserialize(item,context)
            for item in data
        ]
        

@cython.final
cdef class ConstantField(Field):
    """
    A field that always outputs a fixed value.

    :param constant: The value to return.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    default_error_messages = {
        'constant': 'Must be "{constant}".',
        'None': 'Must be None.'
    }
    def __init__(self,constant,**kwargs):
        self.constant = constant
        super().__init__(**kwargs)
        assert "allow_null" not in kwargs
            
    cpdef inline deserialize(self,data,dict context):
        if data != self.constant:
            if self.constant is None:
                raise self.raise_if_fail("None")
            raise self.raise_if_fail("constant",constant=self.constant)
        return data
    
    
@cython.final
cdef class RecursiveField(Field):
    """
    A field that recursively validates its data.

    :param many: Whether the field is a collection of items.
    :param context: The context passed to the field's :meth:`run_validation`.
    :param only: A tuple or list of field names to include. 
    :param exclude: A tuple or list of field names to exclude.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    def __init__(self,**kwargs):
        self.many = kwargs.pop('many',False)
        self.context = kwargs.pop('context',{})
        self.only = kwargs.pop('only',None)
        self.exclude = kwargs.pop('exclude',None)
        if self.only is not None and self.exclude is not None :
            raise OnlyAndExcludeError('You should use either "only" or "exclude"')
        if self.only is not None and not is_collection(self.only):
            raise StringNotCollectionError('"only" should be a list of strings')
        if self.exclude is not None and not is_collection(self.exclude):
            raise StringNotCollectionError('"exclude" should be a list of strings')
        super().__init__(**kwargs)
        
    cpdef inline serialize(self,value,dict context):
        if self.only :
            serializer = self.root.__class__(value,many=self.many,only=self.only,context=self.context)
        elif self.exclude :
            serializer = self.root.__class__(value,many=self.many,exclude=self.exclude,context=self.context)
        else:
            serializer = self.root.__class__(value,many=self.many,context=self.context)
        return serializer.data


@cython.final
cdef class MethodField(Field):
    """
    A field that calls a method on the serializer instead of simply returning a value.

    :param method_name: The name of the method to call.
    :param kwargs: The same keyword arguments that :class:`Field` receives.
    """
    is_method_field = True

    def __init__(self,method_name=None,**kwargs):
        kwargs['read_only']= True
        kwargs['required'] = False
        self.method_name = method_name
        super().__init__(**kwargs)
        
    cpdef inline method_getter(self,field_name, root) :
        if self.method_name is None:
            self.method_name = 'get_{0}'.format(field_name)
        return getattr(root, self.method_name)


from drf_turbo.utils import get_execption_detail

cdef class DrfTurboException(Exception):
    """Base class for all fast_rest-related errors."""
    status_code = 500
    default_detail = 'A server error occurred.'
    default_code = 'error'
    
    def __cinit__(self, detail=None, code=None) -> None:
        if detail is None:
            detail = self.default_detail
        if code is None:
            code = self.default_code
        
        self.detail = get_execption_detail(detail)
        
    def __str__(self) -> str:
        return str(self.detail)

cdef class ValidationError(DrfTurboException):
    default_detail = 'Invalid input.'
    default_code = 'invalid'

    def __cinit__(self, detail=None, code=None) -> None:
        if detail is None:
            detail = self.default_detail
        if code is None:
            code = self.default_code
            
        if isinstance(detail, tuple):
            detail = list(detail)
        elif not isinstance(detail, dict) and not isinstance(detail, list):
            detail = [detail]

        self.detail = get_execption_detail(detail)
        
class StringNotCollectionError(DrfTurboException,TypeError):
    pass


class OnlyAndExcludeError(DrfTurboException):
    pass

            
class ParseError(DrfTurboException):
    status_code = 400
    default_detail = 'Malformed request.'
    default_code = 'parse_error'



from django.test import TestCase
import drf_turbo as dt


class TestJsonResponse(TestCase):
    def test_json_response(self):
        """
        Test json response
        """
        resp = dt.JSONResponse({'a': 'b'})
        self.assertEqual(resp.content, b'{"a":"b"}')
        
    def test_json_response_with_status(self):
        """
        Test json response with status
        """
        resp = dt.JSONResponse({'a': 'b'}, status=400)
        self.assertEqual(resp.status_code, 400)
        
        
    def test_json_response_with_content_type(self):
        """
        Test json response with content type
        """
        resp = dt.JSONResponse({'a': 'b'}, content_type='application/json')
        self.assertEqual(resp['Content-Type'], 'application/json')
        
        
    def test_json_response_with_content_type_with_charset(self):
        """
        Test json response with content type with charset
        """
        resp = dt.JSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8')
        
        
    def test_json_response_with_content_type_with_charset_and_encoding(self):
        """
        Test json response with content type with charset and encoding
        """
        resp = dt.JSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8')
        
        
    def test_json_response_with_content_type_with_charset_and_encoding_and_encoding_errors(self):
        """
        Test json response with content type with charset and encoding and encoding errors
        """
        resp = dt.JSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore')
        
        
    def test_json_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors(self):
        """
        Test json response with content type with charset and encoding and encoding errors and charset errors
        """
        resp = dt.JSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore')
        
        
    def test_json_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent(self):
        """
        Test json response with content type with charset and encoding and encoding errors and charset errors and indent
        """
        resp = dt.JSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4')
        
        
    def test_json_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent_and_separators(self):
        """
        Test json response with content type with charset and encoding and encoding errors and charset errors and indent and separators
        """
        resp = dt.JSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :)')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :)')
        
        
    def test_json_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent_and_separators_and_sort_keys(self):
        """
        Test json response with content type with charset and encoding and encoding errors and charset errors and indent and separators and sort keys
        """
        resp = dt.JSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :); sort-keys=true')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :); sort-keys=true')
        
        

class TestORJSONResponse(TestCase):
    def test_orjson_response(self):
        """
        Test orjson response
        """
        resp = dt.ORJSONResponse({'a': 'b'})
        self.assertEqual(resp.content, b'{"a":"b"}')
        
    def test_orjson_response_with_status(self):
        """
        Test orjson response with status
        """
        resp = dt.ORJSONResponse({'a': 'b'}, status=400)
        self.assertEqual(resp.status_code, 400)
        
        
    def test_orjson_response_with_content_type(self):
        """
        Test orjson response with content type
        """
        resp = dt.ORJSONResponse({'a': 'b'}, content_type='application/json')
        self.assertEqual(resp['Content-Type'], 'application/json')
        
        
    def test_orjson_response_with_content_type_with_charset(self):
        """
        Test orjson response with content type with charset
        """
        resp = dt.ORJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8')
        
    def test_orjson_response_with_content_type_with_charset_and_encoding(self):
        """
        Test orjson response with content type with charset and encoding
        """
        resp = dt.ORJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8')
        
        
    def test_orjson_response_with_content_type_with_charset_and_encoding_and_encoding_errors(self):
        """
        Test orjson response with content type with charset and encoding and encoding errors
        """
        resp = dt.ORJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore')
        
    def test_orjson_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors(self):
        """
        Test orjson response with content type with charset and encoding and encoding errors and charset errors
        """
        resp = dt.ORJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore')
        
        
    def test_orjson_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent(self):
        """
        Test orjson response with content type with charset and encoding and encoding errors and charset errors and indent
        """
        resp = dt.ORJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4')
        
        
    def test_orjson_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent_and_separators(self):
        """
        Test orjson response with content type with charset and encoding and encoding errors and charset errors and indent and separators
        """
        resp = dt.ORJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :)')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :)')
        
        
    def test_orjson_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent_and_separators_and_sort_keys(self):
        resp = dt.ORJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :); sort-keys=true') 
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :); sort-keys=true')
        
        
class TestUJSONResponse(TestCase):
    def test_ujson_response(self):
        """
        Test ujson response
        """
        resp = dt.UJSONResponse({'a': 'b'})
        self.assertEqual(resp.content, b'{"a":"b"}')
        
    def test_ujson_response_with_status(self):
        """
        Test ujson response with status
        """
        resp = dt.UJSONResponse({'a': 'b'}, status=400)
        self.assertEqual(resp.status_code, 400)
        
        
    def test_ujson_response_with_content_type(self):
        """
        Test ujson response with content type
        """
        resp = dt.UJSONResponse({'a': 'b'}, content_type='application/json')
        self.assertEqual(resp['Content-Type'], 'application/json')
        
        
    def test_ujson_response_with_content_type_with_charset(self):
        """
        Test ujson response with content type with charset
        """
        resp = dt.UJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8')
        
    def test_ujson_response_with_content_type_with_charset_and_encoding(self):
        """
        Test ujson response with content type with charset and encoding
        """
        resp = dt.UJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8')
        
        
    def test_ujson_response_with_content_type_with_charset_and_encoding_and_encoding_errors(self):
        """
        Test ujson response with content type with charset and encoding and encoding errors
        """
        resp = dt.UJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore')
        
    def test_ujson_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors(self):
        """
        Test ujson response with content type with charset and encoding and encoding errors and charset errors
        """
        resp = dt.UJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore')
        
    def test_ujson_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent(self):
        """
        Test ujson response with content type with charset and encoding and encoding errors and charset errors and indent
        """
        resp = dt.UJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4')
        
        
    def test_ujson_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent_and_separators(self):
        """
        Test ujson response with content type with charset and encoding and encoding errors and charset errors and indent and separators
        """
        resp = dt.UJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :)')
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :)')
        
    def test_ujson_response_with_content_type_with_charset_and_encoding_and_encoding_errors_and_charset_errors_and_indent_and_separators_and_sort_keys(self):
        resp = dt.UJSONResponse({'a': 'b'}, content_type='application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :); sort-keys=true') 
        self.assertEqual(resp['Content-Type'], 'application/json; charset=UTF-8; encoding=UTF-8; encoding-errors=ignore; charset-errors=ignore; indent=4; separators=(, :); sort-keys=true')
        
        
class TestSuccessResponse(TestCase):
    def test_success_response(self):
        """
        Test success response
        """
        resp = dt.SuccessResponse({'a': 'b'})   
        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.content, b'{"message":"Success","data":{"a":"b"},"error":false}')
        
        
class TestErrorResponse(TestCase):
    def test_error_response(self):
        """
        Test error response
        """
        resp = dt.ErrorResponse({'a': 'b'})
        self.assertEqual(resp.status_code, 400)
        self.assertEqual(resp.content, b'{"message":"Bad request","data":{"a":"b"},"error":true}')



from django.db import models
from drf_turbo.fields import (
    Field,IntField,BoolField,StrField,DateField,TimeField,DateTimeField,EmailField,FileField,FloatField,URLField,IPField,UUIDField,RelatedField,ManyRelatedField,ChoiceField,DecimalField,ArrayField,JSONField
)

try:
    from django.contrib.postgres import fields as postgres_fields
    
except ImportError:
    postgres_fields = None

class SerializerMetaclass(type):
    @classmethod
    def _get_fields(cls,bases,attrs):
        fields = [(field_name, attrs.pop(field_name))
                for field_name, obj in list(attrs.items())
                if isinstance(obj, Field)]
        for base in reversed(bases):
            if hasattr(base, '_fields'):
                fields = list(
                    getattr(base,'_fields').items()
                ) + fields
        return dict(fields)



    def __new__(cls, name, bases, attrs):
        attrs['_fields'] = cls._get_fields(bases, attrs)
        return super().__new__(cls, name, bases, attrs)


class ModelSerializerMetaclass(SerializerMetaclass):

    TYPE_MAPPING = {
        models.AutoField: IntField,
        models.BigIntegerField: IntField,
        models.BooleanField: BoolField,
        models.CharField: StrField,
        models.DateField: DateField,
        models.TimeField : TimeField,
        models.DateTimeField: DateTimeField,
        models.EmailField: EmailField,
        models.FileField: FileField,
        models.FloatField: FloatField,
        models.ImageField: FileField,
        models.IntegerField: IntField,
        models.NullBooleanField: BoolField,
        models.PositiveIntegerField: IntField,
        models.PositiveSmallIntegerField: IntField,
        models.SmallIntegerField: IntField,
        models.TextField: StrField,
        models.URLField: URLField,
        models.GenericIPAddressField: IPField,
        models.JSONField : JSONField,
        models.UUIDField : UUIDField,
    
    }
    
    if postgres_fields is not None:
        TYPE_MAPPING[postgres_fields.ArrayField] = ArrayField
        TYPE_MAPPING[postgres_fields.JSONField] = JSONField

    @staticmethod
    def _get_implicit_fields(model_fields,fields,exclude):

        if fields == '__all__':
            fields = model_fields
        elif fields and isinstance(fields,(list,tuple)):
            fields = [
                field for field in model_fields
                if field.name in fields
            ]

        elif not fields and exclude:
            fields = [
                field for field in model_fields
                if field.name not in exclude
            ]
        # this implicitly handles the case when `fields` is set and `exclude`
        # isn't. Then all fields declared will be returned without any
        # modification.
        return fields

    @staticmethod
    def _filter_fields(cls,declared_fields,explicit_fields,implicit_fields):
        for field in implicit_fields :
            if field.name not in explicit_fields:
                klass = field.__class__
                if issubclass(klass,(models.ForeignKey,models.OneToOneField)):
                    field_obj = RelatedField()
                    field_obj.queryset = field.related_model.objects
                elif issubclass(klass,models.ManyToManyField) :
                    field_obj = ManyRelatedField()
                    child_obj = RelatedField()
                    child_obj.queryset = field.related_model.objects
                    field_obj.child_relation = child_obj
                elif issubclass(klass,models.DecimalField) : 
                    field_obj = DecimalField(max_digits=field.max_digits,decimal_places=field.decimal_places)
                else:
                    try:
                        field.get_choices()
                        field_obj = ChoiceField(choices=field.choices)
                    except :
                        field_obj = cls.TYPE_MAPPING.get(klass,Field)()
                
                field_obj.attr = field.name
                field_obj.help_text = str(field.help_text)
                field_obj.validators = field.validators
                if issubclass(klass, models.AutoField) or not field.editable :
                    field_obj.read_only = True
                if field.has_default() or field.blank or field.null:
                    field_obj.required = False
                if field.get_internal_type() == 'TextField':
                    field_obj.default_value = ''
                if field.has_default():
                    field_obj.default_value = field.default.value if hasattr(field.default,'value') else field.default
                if getattr(field, 'auto_now', False):
                    field_obj.default_value = field.auto_now
                if getattr(field, 'auto_now_add', False):
                    field_obj.default_value = field.auto_now_add

                declared_fields[field.name] = field_obj

        return declared_fields
    
    
    @staticmethod
    def _read_only_fields(read_only_fields,declared_fields):
        if read_only_fields:
            if not isinstance(read_only_fields, (list, tuple)):
                raise TypeError(
                    'The `read_only_fields` option must be a list or tuple. '
                    'Got %s.' % type(read_only_fields).__name__
                )
            for field in read_only_fields:
                declared_fields[field].read_only = True
                declared_fields[field].required = False


    @staticmethod
    def _write_only_fields(write_only_fields,declared_fields):
        if write_only_fields:
            if not isinstance(write_only_fields, (list, tuple)):
                raise TypeError(
                    'The `write_only_fields` option must be a list or tuple. '
                    'Got %s.' % type(write_only_fields).__name__
                )
            for field in write_only_fields:
                declared_fields[field].write_only = True



    def __new__(cls, name, bases, attrs):
        klass =  super().__new__(cls, name, bases, attrs)
        declared_fields = attrs['_fields']
        meta = getattr(klass, 'Meta', None)
        if meta:
            model = getattr(meta, 'model', None)
            fields = getattr(meta, 'fields', None)
            exclude = getattr(meta, 'exclude', None)
            read_only_fields = getattr(meta,'read_only_fields',None)
            write_only_fields = getattr(meta,'write_only_fields',None)
            if not model:
                raise RuntimeError(
                    'If you specifiy a Meta class, you need to atleast specify a model'
                )
            if not fields and not exclude:
                raise RuntimeError(
                    'You need to specifiy either `fields` or `exclude` in Meta'
                )
            if fields and exclude:
                raise RuntimeError(
                    '`fields` and `exclude` prohibit each other.'
                )
           
                    
            if hasattr(model, "_meta"):
                # Django models
                model_fields = [field for field in model._meta.fields]
                many_to_many_fields = [field for field in model._meta.many_to_many]
                model_fields.extend(many_to_many_fields)
                implicit_fields = cls._get_implicit_fields(
                    model_fields, fields, exclude
                )
                explicit_fields = declared_fields.keys()
                declared_fields = cls._filter_fields(cls,
                    declared_fields, explicit_fields, implicit_fields
                )
                cls._read_only_fields(read_only_fields,declared_fields)
                cls._write_only_fields(write_only_fields,declared_fields)
                
                klass._model_fields = model_fields
        klass._fields = declared_fields
        

        return klass




import pytest
from django.test import TestCase
import drf_turbo as dt


class JSONRendererTests(TestCase):
    """
    Tests specific to the JSON Renderer
    """

    def test_render_lazy_strings(self):
        """
        JSONRenderer should deal with lazy translated strings.
        """
        ret = dt.JSONRenderer().render('test')
        self.assertEqual(ret, b'"test"')

    def test_render_none(self):
        """
        Renderer should deal with None as ''.
        """
        ret = dt.JSONRenderer().render(None)
        self.assertEqual(ret, b'')
        
    def test_render_dict(self):
        """
        Test render dict
        """
        ret = dt.JSONRenderer().render({'a': 'b'})
        self.assertEqual(ret, b'{"a":"b"}')
        
        
    def test_render_list(self):
        """
        Test render list
        """
        ret = dt.JSONRenderer().render(['a', 'b'])
        self.assertEqual(ret, b'["a","b"]')
        
        
    def test_render_int(self):
        """
        Test render int
        """
        ret = dt.JSONRenderer().render(1)
        self.assertEqual(ret, b'1')
        
        
    def test_render_float(self):
        """
        Test render float
        """
        ret = dt.JSONRenderer().render(1.1)
        self.assertEqual(ret, b'1.1')
        
    def test_render_bool(self):
        """
        Test render bool
        """
        ret = dt.JSONRenderer().render(True)
        self.assertEqual(ret, b'true')
                
class ORJSONRendererTests(TestCase):
    """
    Tests specific to the JSON Renderer
    """

    def test_render_lazy_strings(self):
        """
        ORJSONRenderer should deal with lazy translated strings.
        """
        ret = dt.ORJSONRenderer().render('test')
        self.assertEqual(ret, b'"test"')

    def test_render_none(self):
        """
        Renderer should deal with None as ''.
        """
        ret = dt.ORJSONRenderer().render(None)
        self.assertEqual(ret, b'')
        
    def test_render_dict(self):
        """
        Test render dict
        """
        ret = dt.ORJSONRenderer().render({'a': 'b'})
        self.assertEqual(ret, b'{"a":"b"}')
        
        
    def test_render_list(self):
        """
        Test render list
        """
        ret = dt.ORJSONRenderer().render(['a', 'b'])
        self.assertEqual(ret, b'["a","b"]')
        
        
    def test_render_int(self):
        """
        Test render int
        """
        ret = dt.ORJSONRenderer().render(1)
        self.assertEqual(ret, b'1')
        
        
    def test_render_float(self):
        """
        Test render float
        """
        ret = dt.ORJSONRenderer().render(1.1)
        self.assertEqual(ret, b'1.1')
        
    def test_render_bool(self):
        """
        Test render bool
        """
        ret = dt.ORJSONRenderer().render(True)
        self.assertEqual(ret, b'true')
                
class UJSONRendererTests(TestCase):
    """
    Tests specific to the UJSON Renderer
    """

    def test_render_lazy_strings(self):
        """
        UJSONRenderer should deal with lazy translated strings.
        """
        ret = dt.UJSONRenderer().render('test')
        self.assertEqual(ret, b'"test"')

    def test_render_none(self):
        """
        Renderer should deal with None as ''.
        """
        ret = dt.UJSONRenderer().render(None)
        self.assertEqual(ret, b'')
        
    def test_render_dict(self):
        """
        Test render dict
        """
        ret = dt.UJSONRenderer().render({'a': 'b'})
        self.assertEqual(ret, b'{"a":"b"}')
        
        
    def test_render_list(self):
        """
        Test render list
        """
        ret = dt.UJSONRenderer().render(['a', 'b'])
        self.assertEqual(ret, b'["a","b"]')
        
        
    def test_render_int(self):
        """
        Test render int
        """
        ret = dt.UJSONRenderer().render(1)
        self.assertEqual(ret, b'1')
        
        
    def test_render_float(self):
        """
        Test render float
        """
        ret = dt.UJSONRenderer().render(1.1)
        self.assertEqual(ret, b'1.1')
        
    def test_render_bool(self):
        """
        Test render bool
        """
        ret = dt.UJSONRenderer().render(True)
        self.assertEqual(ret, b'true')
        
        


        


import pytest
from django.test import TestCase
from rest_framework.test import APIRequestFactory
from drf_turbo.exceptions import ParseError
import drf_turbo as dt

class TestJSONParser(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
    
    def test_json_parser(self):
        request = self.factory.post('/', '{"a": "b"}', content_type='application/json')
        assert dt.JSONParser().parse(request) == {'a': 'b'}
        
    def test_json_parser_with_empty_request(self):
        request = self.factory.post('/', '', content_type='application/json')
        with pytest.raises(ParseError):
            dt.JSONParser().parse(request)
        
    def test_json_parser_with_invalid_json(self):   
        request = self.factory.post('/', '{"a": "b"', content_type='application/json')
        with pytest.raises(ParseError):
            dt.JSONParser().parse(request)
                        
            

class TestORJSONParser(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
    
    def test_orjson_parser(self):
        request = self.factory.post('/', '{"a": "b"}', content_type='application/json')
        assert dt.ORJSONParser().parse(request) == {'a': 'b'}
        
    def test_orjson_parser_with_empty_request(self):
        request = self.factory.post('/', '', content_type='application/json')
        with pytest.raises(ParseError):
            dt.ORJSONParser().parse(request)
        
    def test_orjson_parser_with_invalid_json(self):   
        request = self.factory.post('/', '{"a": "b"', content_type='application/json')
        with pytest.raises(ParseError):
            dt.ORJSONParser().parse(request)



class TestUJSONParser(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()
    
    def test_ujson_parser(self):
        request = self.factory.post('/', '{"a": "b"}', content_type='application/json')
        assert dt.UJSONParser().parse(request) == {'a': 'b'}
        
    def test_ujson_parser_with_empty_request(self):
        request = self.factory.post('/', '', content_type='application/json')
        with pytest.raises(ParseError):
            dt.UJSONParser().parse(request)
        
    def test_ujson_parser_with_invalid_json(self):   
        request = self.factory.post('/', '{"a": "b"', content_type='application/json')
        with pytest.raises(ParseError):
            dt.UJSONParser().parse(request)




from django.http.multipartparser import parse_header
from rest_framework.utils import encoders, json
from rest_framework.compat import (
    INDENT_SEPARATORS, LONG_SEPARATORS, SHORT_SEPARATORS
)   
try:
    import orjson
except ImportError:
    orjson = None  

try:
    import ujson
except ImportError:
    ujson = None  


cimport cython

cdef zero_as_none(value):
    return None if value == 0 else value

cdef class BaseRenderer:
    """
    All renderers should extend this class, setting the `media_type`
    and `format` attributes, and override the `.render()` method.
    """
    media_type = None
    format = None
    charset = 'utf-8'
    render_style = 'text'

    cpdef bytes render(self,data, accepted_media_type=None,renderer_context=None):
        raise NotImplementedError('Renderer class requires .render() to be implemented')


@cython.final
cdef class JSONRenderer(BaseRenderer):
    """
    Renderer which serializes to JSON.
    """
    media_type = 'application/json'
    format = 'json'
    encoder_class = encoders.JSONEncoder
    ensure_ascii = False
    compact = True
    strict = True
    charset = None

    cpdef inline get_indent(self,unicode accepted_media_type, dict renderer_context):
        if accepted_media_type:
            base_media_type, params = parse_header(accepted_media_type.encode('ascii'))
            try:
                return zero_as_none(max(min(int(params['indent']), 8), 0))
            except (KeyError, ValueError, TypeError):
                pass
        return renderer_context.get('indent', None)

    cpdef inline bytes render(self, data, accepted_media_type=None, renderer_context=None):
        """
        Render `data` into JSON, returning a bytestring.
        """
        cdef basestring ret
        if data is None:
            return b''

        renderer_context = renderer_context or {}
        indent = self.get_indent(accepted_media_type, renderer_context)

        if indent is None:
            separators = SHORT_SEPARATORS if self.compact else LONG_SEPARATORS
        else:
            separators = INDENT_SEPARATORS

        ret = json.dumps(
            data, cls=self.encoder_class,
            indent=indent, ensure_ascii=self.ensure_ascii,
            allow_nan=not self.strict, separators=separators
        )

        ret = ret.replace('\u2028', '\\u2028').replace('\u2029', '\\u2029')
        return ret.encode()

@cython.final
cdef class UJSONRenderer(BaseRenderer):
    """
    Renderer which serializes to JSON.
    """
    media_type = 'application/json'
    format = 'json'
    ensure_ascii = False
    escape_forward_slashes= False   
    encode_html_chars= False
    charset = None


    cpdef inline get_indent(self,unicode accepted_media_type, dict renderer_context):
        cdef dict params
        if accepted_media_type:
            _, params = parse_header(accepted_media_type.encode('ascii'))
            try:
                return zero_as_none(max(min(int(params['indent']), 8), 0))
            except (KeyError, ValueError, TypeError):
                pass

        return renderer_context.get('indent', None)

    cpdef inline bytes render(self, data,accepted_media_type=None,renderer_context=None):
        """
        Render `data` into JSON, returning a bytestring.
        """
        assert ujson is not None, "ujson must be installed to use UJSONRenderer"
        cdef basestring ret
        if data is None:
            return b''

        renderer_context = renderer_context or {}
        indent = self.get_indent(accepted_media_type, renderer_context)


        ret = ujson.dumps(
            data,
            indent=indent or 0, 
            ensure_ascii=self.ensure_ascii,
            encode_html_chars=self.encode_html_chars,
            escape_forward_slashes=self.escape_forward_slashes,
        )
        ret = ret.replace("\u2028", "\\u2028").replace("\u2029", "\\u2029")
        return bytes(ret.encode("utf-8"))

@cython.final
cdef class ORJSONRenderer(BaseRenderer):
    """
    Renderer which serializes to JSON.
    """

    media_type = "application/json"
    format = "json"
    charset = None
        


    cpdef inline bytes render(self, data,accepted_media_type=None,renderer_context=None):
        """
        Render `data` into JSON, returning a bytestring.
        """
        assert orjson is not None, "orjson must be installed to use ORJSONParser"
        if data is None:
            return b''

        return orjson.dumps(
            data,
            option = orjson.OPT_SERIALIZE_UUID | \
                orjson.OPT_SERIALIZE_NUMPY | \
                orjson.OPT_SERIALIZE_DATACLASS | \
                orjson.OPT_NON_STR_KEYS,
        )



from drf_turbo.fields cimport Field
cimport cython_metaclass


cdef class BaseSerializer(Field) :
    cdef:
        object instance
        public bint many
        object data
        public dict context
        readonly object only
        readonly object exclude
        public bint partial 

    cpdef bint is_valid(self,bint raise_exception=*) except -1
    cpdef dict get_initial_data(self)




cdef class Serializer(BaseSerializer):
    cdef inline dict _parse_nested_fields(self,object fields)
    cdef inline void _select_nested_fields(self,Serializer serializer,object fields,basestring action,bint is_nested=*)
    cdef inline object _fields_to_include(self,Serializer serializer,object fields)
    cdef inline object _fields_to_exclude(self,Serializer serializer,object fields,bint is_nested)
    cdef dict _serialize(self,object instance,dict fields)
    cpdef serialize(self,instance,dict context)
    cdef dict _deserialize(self, object data,dict fields)
    cpdef deserialize(self,object data,dict context)
    cpdef run_validation(self,object data,dict context)
    cpdef validate(self,object data)



import pytest
from collections import ChainMap
from collections.abc import Mapping
import drf_turbo as dt
import pickle
import re
from drf_turbo.exceptions import OnlyAndExcludeError


class TestSerializer:
    def setup(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()
        self.Serializer = ExampleSerializer
        self.context = {}

    def test_valid_serializer(self):
        serializer = self.Serializer(data={'char': 'abc', 'integer': 123})
        assert serializer.is_valid()
        assert serializer.validated_data == {'char': 'abc', 'integer': 123}
        assert serializer.data == {'char': 'abc', 'integer': 123}
        assert serializer.errors == {}

    def test_invalid_serializer(self):
        serializer = self.Serializer(data={'char': 'abc'})
        assert not serializer.is_valid()
        assert serializer.validated_data == {}
        assert serializer.data == {'char': 'abc'}
        assert serializer.errors == {'integer': ['This field is required.']}

    def test_invalid_datatype(self):
        serializer = self.Serializer(data=[{'char': 'abc'}])
        assert not serializer.is_valid()
        assert serializer.validated_data == {}
        # assert serializer.data == {}
        assert serializer.errors == ['Invalid data type: list']

    def test_partial_validation(self):
        serializer = self.Serializer(data={'char': 'abc'}, partial=True)
        assert serializer.is_valid()
        assert serializer.validated_data == {'char': 'abc'}
        assert serializer.errors == {}

    def test_empty_serializer(self):
        serializer = self.Serializer()
        assert serializer.data == {'char': '', 'integer': None}

    def test_missing_attribute_during_serialization(self):
        class MissingAttributes:
            pass
        instance = MissingAttributes()
        serializer = self.Serializer(instance)
        with pytest.raises(AttributeError):
            serializer.data

    def test_data_access_before_save_raises_error(self):
        def create(validated_data):
            return validated_data
        serializer = self.Serializer(data={'char': 'abc', 'integer': 123})
        serializer.create = create
        assert serializer.is_valid()
        assert serializer.data == {'char': 'abc', 'integer': 123}
        with pytest.raises(AssertionError):
            serializer.save()

    def test_validate_none_data(self):
        data = None
        serializer = self.Serializer(data=data)
        assert not serializer.is_valid()
        assert serializer.errors == ['Invalid data type: NoneType']

    def test_serialize_chainmap(self):
        data = ChainMap({'char': 'abc'}, {'integer': 123})
        serializer = self.Serializer(data=data) 
        assert serializer.is_valid()
        assert serializer.validated_data == {'char': 'abc', 'integer': 123}
        assert serializer.errors == {}

    def test_serialize_custom_mapping(self):
        class SinglePurposeMapping(Mapping):
            def __getitem__(self, key):
                return 'abc' if key == 'char' else 123

            def __iter__(self):
                yield 'char'
                yield 'integer'

            def __len__(self):
                return 2

        serializer = self.Serializer(data=SinglePurposeMapping())
        assert serializer.is_valid()
        assert serializer.validated_data == {'char': 'abc', 'integer': 123}
        assert serializer.errors == {}

    def test_custom_deserialize(self):
        """
        deserialize() is expected to return a dict, but subclasses may
        return application specific type.
        """
        class Point:
            def __init__(self, srid, x, y):
                self.srid = srid
                self.coords = (x, y)

        # Declares a serializer that converts data into an object
        class NestedPointSerializer(dt.Serializer):
            longitude = dt.FloatField(attr='x')
            latitude = dt.FloatField(attr='y')

            def deserialize(self, data,context):
                kwargs = super().deserialize(data,context)
                return Point(srid=4326, **kwargs)

        serializer = NestedPointSerializer(data={'longitude': 6.958307, 'latitude': 50.941357})
        assert serializer.is_valid()
        assert isinstance(serializer.validated_data, Point)
        assert serializer.validated_data.srid == 4326
        assert serializer.validated_data.coords[0] == 6.958307
        assert serializer.validated_data.coords[1] == 50.941357
        assert serializer.errors == {}

    def test_iterable_validators(self):
        """
        Ensure `validators` parameter is compatible with reasonable iterables.
        """
        data = {'char': 'abc', 'integer': 123}

        for validators in ([], (), set()):
            class ExampleSerializer(dt.Serializer):
                char = dt.StrField(validators=validators)
                integer = dt.IntField()

            serializer = ExampleSerializer(data=data)
            assert serializer.is_valid()
            assert serializer.validated_data == data
            assert serializer.errors == {}

        def raise_exception(value):
            raise dt.ValidationError('Raised error')

        for validators in ([raise_exception], (raise_exception,), {raise_exception}):
            class ExampleSerializer(dt.Serializer):
                char = dt.StrField(validators=validators)
                integer = dt.IntField()

            serializer = ExampleSerializer(data=data)
            assert not serializer.is_valid()
            assert serializer.data == data
            assert serializer.validated_data == {}

    def test_only_fields(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()

        serializer = ExampleSerializer({'char': 'abc', 'integer': 123}, only=('char',))
        assert serializer.data == {'char': 'abc'}


    def test_exclude_fields(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()

        serializer = ExampleSerializer({'char': 'abc', 'integer': 123}, exclude=('char',))
        assert serializer.data == {'integer': 123}
        
    def test_not_allowed_only_and_exclude_fields(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()

        with pytest.raises(OnlyAndExcludeError):
            
            ExampleSerializer({'char': 'abc', 'integer': 123}, only=('char',), exclude=('integer',))
            
            
    def test_nested_only_fields(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()

        class NestedSerializer(dt.Serializer):  
            example = ExampleSerializer()

        serializer = NestedSerializer({'example': {'char': 'abc', 'integer': 123}}, only=('example__char',))
        assert serializer.data == {'example': {'char': 'abc'}}
        
        
    def test_nested_exclude_fields(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()

        class NestedSerializer(dt.Serializer):  
            example = ExampleSerializer()

        serializer = NestedSerializer({'example': {'char': 'abc', 'integer': 123}}, exclude=('example__char',))
        assert serializer.data == {'example': {'integer': 123}}
        
        
    def test_nested_only_inheritance(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()
            other = dt.StrField()

        class NestedSerializer(dt.Serializer):  
            foo = dt.StrField()
            bar = dt.IntField()
            example = ExampleSerializer(only=('char','other'))

        serializer = NestedSerializer({'foo' : 'foo','bar' : 123, 'example': {'char': 'abc', 'integer': 123 , 'other' : 'other'}}, only=('foo','example__other'))
        assert serializer.data == {'example': {'other': 'other'}, 'foo': 'foo'}
        
        
    def test_nested_exclude_inheritance(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()
            other = dt.StrField()

        class NestedSerializer(dt.Serializer):  
            foo = dt.StrField()
            bar = dt.IntField()
            example = ExampleSerializer(exclude=('char',))

        serializer = NestedSerializer({'foo' : 'foo','bar' : 123, 'example': {'char': 'abc', 'integer': 123 , 'other' : 'other'}}, exclude=('foo','example__other'))
        assert serializer.data == {'example': {'integer': 123}, 'bar': 123}
        
        
        
class TestValidateMethod:
    def test_non_field_error_validate_method(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()

            def validate(self, attrs):
                raise dt.ValidationError('Non field error')

        serializer = ExampleSerializer(data={'char': 'abc', 'integer': 123})
        assert not serializer.is_valid()
        assert serializer.errors == ['Non field error']

    def test_field_error_validate_method(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField()
            integer = dt.IntField()

            def validate(self, attrs):
                raise dt.ValidationError({'char': 'Field error'})

        serializer = ExampleSerializer(data={'char': 'abc', 'integer': 123})
        assert not serializer.is_valid()
        assert serializer.errors == {'char': 'Field error'}


class MockObject:
    def __init__(self, **kwargs):
        self._kwargs = kwargs
        for key, val in kwargs.items():
            setattr(self, key, val)

    def __str__(self):
        kwargs_str = ', '.join([
            '%s=%s' % (key, value)
            for key, value in sorted(self._kwargs.items())
        ])
        return '<MockObject %s>' % kwargs_str




class TestNotRequiredOutput:
    def test_not_required_output_for_dict(self):
        """
        'required=False' should allow a dictionary key to be missing in output.
        """
        class ExampleSerializer(dt.Serializer):
            omitted = dt.StrField(required=False)
            included = dt.StrField()

        serializer = ExampleSerializer(data={'included': 'abc'})
        serializer.is_valid()
        assert serializer.data == {'included': 'abc'}

    def test_not_required_output_for_object(self):
        """
        'required=False' should allow an object attribute to be missing in output.
        """
        class ExampleSerializer(dt.Serializer):
            omitted = dt.StrField(required=False)
            included = dt.StrField()

            def create(self, validated_data):
                return MockObject(**validated_data)

        serializer = ExampleSerializer(data={'included': 'abc'})
        serializer.is_valid()
        serializer.save()
        assert serializer.data == {'included': 'abc'}


class TestDefaultOutput:
    def setup(self):
        class ExampleSerializer(dt.Serializer):
            has_default = dt.StrField(default_value='x',required=False)
            has_default_callable = dt.StrField(default_value=lambda: 'y',required=False)
            no_default = dt.StrField()
        self.Serializer = ExampleSerializer

    def test_default_used_for_dict(self):
        """
        'default="something"' should be used if dictionary key is missing from input.
        """
        serializer = self.Serializer({'no_default': 'abc'})
        assert serializer.data == {'has_default': 'x', 'has_default_callable': 'y', 'no_default': 'abc'}

    def test_default_used_for_object(self):
        """
        'default="something"' should be used if object attribute is missing from input.
        """
        instance = MockObject(no_default='abc')
        serializer = self.Serializer(instance)
        assert serializer.data == {'has_default': 'x', 'has_default_callable': 'y', 'no_default': 'abc'}

    def test_default_not_used_when_in_dict(self):
        """
        'default="something"' should not be used if dictionary key is present in input.
        """
        serializer = self.Serializer({'has_default': 'def', 'has_default_callable': 'ghi', 'no_default': 'abc'})
        assert serializer.data == {'has_default': 'def', 'has_default_callable': 'ghi', 'no_default': 'abc'}

    def test_default_not_used_when_in_object(self):
        """
        'default="something"' should not be used if object attribute is present in input.
        """
        instance = MockObject(has_default='def', has_default_callable='ghi', no_default='abc')
        serializer = self.Serializer(instance)
        assert serializer.data == {'has_default': 'def', 'has_default_callable': 'ghi', 'no_default': 'abc'}
        
        
    def test_default_for_dotted_source(self):
        """
        'default="something"' should be used when a traversed attribute is missing from input.
        """
        class Serializer(dt.Serializer):
            traversed = dt.StrField(default_value='x', attr='traversed.attr',required=False)

        assert Serializer({}).data == {'traversed': 'x'}
        assert Serializer({'traversed': {}}).data == {'traversed': 'x'}
        assert Serializer({'traversed': None}).data == {'traversed': 'x'}

        assert Serializer({'traversed': {'attr': 'abc'}}).data == {'traversed': 'abc'}


    def test_default_for_nested_serializer(self):
        class NestedSerializer(dt.Serializer):
            a = dt.StrField(default_value='1',required=False)
            c = dt.StrField(default_value='2', attr='b.c',required=False)

        class Serializer(dt.Serializer):
            nested = NestedSerializer()

        assert Serializer({'nested': None}).data == {'nested': None}
        assert Serializer({'nested': {}}).data == {'nested': {'a': '1', 'c': '2'}}
        assert Serializer({'nested': {'a': '3', 'b': {}}}).data == {'nested': {'a': '3', 'c': '2'}}
        assert Serializer({'nested': {'a': '3', 'b': {'c': '4'}}}).data == {'nested': {'a': '3', 'c': '4'}}

    def test_default_for_allow_null(self):
        """
        Without an explicit default, allow_null implies default=None when serializing. #5518 #5708
        """
        class Serializer(dt.Serializer):
            foo = dt.StrField()
            bar = dt.StrField(attr='foo.bar', allow_null=True)
            optional = dt.StrField(required=False, allow_null=True)

        # allow_null=True should imply default=None when serializing:
        assert Serializer({'foo': None}).data == {'foo': None, 'bar': None, 'optional': None, }

 
class TestCacheSerializerData:
    def test_cache_serializer_data(self):
        """
        Caching serializer data with pickle will drop the serializer info,
        but does preserve the data itself.
        """
        class ExampleSerializer(dt.Serializer):
            field1 = dt.StrField()
            field2 = dt.StrField()

        serializer = ExampleSerializer({'field1': 'a', 'field2': 'b'})
        pickled = pickle.dumps(serializer.data)
        data = pickle.loads(pickled)
        assert data == {'field1': 'a', 'field2': 'b'}


class TestDefaultInclusions:
    def setup(self):
        class ExampleSerializer(dt.Serializer):
            char = dt.StrField(default_value='abc',required=False)
            integer = dt.IntField()
        self.Serializer = ExampleSerializer

    def test_default_should_included_on_create(self):
        serializer = self.Serializer(data={'integer': 456})
        assert serializer.is_valid()
        assert serializer.validated_data == {'char': 'abc', 'integer': 456}
        assert serializer.errors == {}

    def test_default_should_be_included_on_update(self):
        instance = MockObject(char='def', integer=123)
        serializer = self.Serializer(instance, data={'integer': 456})
        assert serializer.is_valid()
        assert serializer.validated_data == {'char': 'abc', 'integer': 456}
        assert serializer.errors == {}

    def test_default_should_not_be_included_on_partial_update(self):
        instance = MockObject(char='def', integer=123)
        serializer = self.Serializer(instance, data={'integer': 456}, partial=True)
        assert serializer.is_valid()
        assert serializer.validated_data == {'integer': 456}
        assert serializer.errors == {}


class TestSerializerValidationWithCompiledRegexField:
    def setup(self):
        class ExampleSerializer(dt.Serializer):
            name = dt.RegexField(re.compile(r'\d'), required=True)
        self.Serializer = ExampleSerializer

    def test_validation_success(self):
        serializer = self.Serializer(data={'name': '2'})
        assert serializer.is_valid()
        assert serializer.validated_data == {'name': '2'}
        assert serializer.errors == {}





from django.conf import settings
from drf_turbo.exceptions import ParseError
import json
import codecs


try:
    import ujson
except ImportError:  
    ujson = None  

try:
    import orjson
except ImportError:
    orjson = None  

cimport cython

cdef class BaseParser:
    """
    All parsers should extend `BaseParser`, specifying a `media_type`
    attribute, and overriding the `.parse()` method.
    """
    media_type = None

    cpdef dict parse(self, stream, media_type=None, parser_context=None):
        """
        Given a stream to read from, return the parsed representation.
        Should return parsed data, or a `DataAndFiles` object consisting of the
        parsed data and files.
        """
        raise NotImplementedError(".parse() must be overridden.")


@cython.final
cdef class JSONParser(BaseParser):
    """
    Parses JSON-serialized data.
    """
    media_type = 'application/json'

    cpdef inline dict parse(self, stream, media_type=None, parser_context=None):
        parser_context = parser_context or {}
        encoding = parser_context.get('encoding', settings.DEFAULT_CHARSET)

        try:
            data = stream.read().decode(encoding)
            return ujson.loads(data)
        except ValueError as exc:
            raise ParseError('JSON parse error - %s' % str(exc))


@cython.final
cdef class UJSONParser(BaseParser):
    """
    Parses JSON-serialized data by ujson parser.
    """

    media_type = "application/json"

    cpdef inline dict parse(self, stream, media_type=None, parser_context=None) :
        assert ujson is not None, "ujson must be installed to use UJSONParser"
        parser_context = parser_context or {}
        encoding = parser_context.get("encoding", settings.DEFAULT_CHARSET)

        try:
            data = stream.read().decode(encoding)
            return ujson.loads(data)
        except ValueError as exc:
            raise ParseError('ORJSON parse error - %s' % str(exc))


@cython.final
cdef class ORJSONParser(BaseParser):
    """
    Parses JSON-serialized data by orjson parser.
    """

    media_type = "application/json"

    cpdef inline dict parse(self, stream, media_type=None, parser_context=None) :
        assert orjson is not None, "orjson must be installed to use ORJSONParser"
        parser_context = parser_context or {}
        encoding = parser_context.get("encoding", settings.DEFAULT_CHARSET)

        try:
            data = stream.read().decode(encoding)
            return orjson.loads(data)
        except ValueError as exc:
            raise ParseError('ORJSON parse error - %s' % str(exc))




cdef object NO_DEFAULT

cdef class SkipField(Exception):
    pass

cdef class Field :
    cdef public :
        str attr
        bint call
        bint required
        bint write_only
        bint read_only
        bint allow_null
        str label
        object default_value
        object initial
        str help_text
        dict style
        str field_name
        object root
        object validators
        dict error_messages
        list attrs

    cpdef serialize(self, value , dict context) 
    cpdef deserialize(self,data, dict context)
    cpdef method_getter(self,field_name, root)
    cpdef void bind(self,basestring name, object root)
    cpdef run_validation(self,object data,dict context)
    cpdef get_initial(self)
    cpdef get_attribute(self, instance,attr=*)
    cpdef get_default_value(self)
    cpdef tuple validate_empty_values(self, data)
    cpdef long validate_or_raise(self,value) except -1


cdef class StrField(Field):
    cdef public:
        allow_blank
        trim_whitespace
        max_length
        min_length

cdef class EmailField(StrField):
    cdef public:
        bint to_lower

cdef class URLField(StrField):
    pass

cdef class RegexField(StrField):
    cdef public:
        regex

cdef class IPField(StrField):
    pass

cdef class PasswordField(StrField):
    pass

cdef class UUIDField(StrField):
    pass

cdef class SlugField(Field):
    cdef public:
        allow_unicode
        
cdef class IntField(Field):
    cdef public :
        max_value
        min_value

cdef class FloatField(Field):
    cdef public :
        max_value
        min_value

cdef class DecimalField(Field):
    cdef public:
        max_digits
        decimal_places
        max_value
        min_value
        rounding
        coerce_to_string
        max_whole_digits
    
    cdef validate_precision(self,value)
    cdef quantize(self,value)

cdef class BoolField(Field):
    pass

cdef class ChoiceField(Field):
    cdef public :
        choices
        choice_strings_to_values
        choice_strings_to_display
        allow_blank

cdef class MultipleChoiceField(ChoiceField):
        cdef public:
            allow_empty
            
cdef class DateTimeField(Field):
    cdef public:
        format
        input_formats
        default_timezone
        timezone

    cpdef get_default_timezone(self)
    cpdef enforce_timezone(self, value)

cdef class DateField(Field):
    cdef public:
        format
        input_formats

cdef class TimeField(Field):
    cdef public:
        format
        input_formats

cdef class FileField(Field):
    cdef public :
        max_length
        allow_empty_file

cdef class ArrayField(Field):
    cdef public :
        child
        allow_empty
        min_items
        max_items
        exact_items

    cpdef run_child_validation(self,data,dict context)

cdef class DictField(Field):
    cdef public :
        child
        allow_empty

    cpdef run_child_validation(self,dict data,dict context)


cdef class JSONField(Field):
    cdef public :
        binary
        encoder
        decoder
        

cdef class RelatedField(Field):
    cdef public:
        queryset

cdef class ManyRelatedField(Field):
    cdef public :
        child_relation
        allow_empty

cdef class ConstantField(Field):
    cdef public :
        constant

cdef class RecursiveField(Field):
    cdef public :
        many
        context
        only
        exclude
    cpdef serialize(self,value,dict context)

cdef class MethodField(Field):
    cdef public :
        method_name




from drf_turbo.serializer import BaseSerializer,Serializer,ModelSerializer
from drf_turbo.fields import (
    Field,StrField,EmailField,URLField,RegexField,IPField,PasswordField,UUIDField,SlugField,IntField,FloatField,DecimalField,BoolField,ChoiceField,MultipleChoiceField,DateTimeField,DateField,TimeField,FileField,ArrayField,DictField,JSONField,RelatedField,ManyRelatedField,ConstantField,RecursiveField,MethodField
)
from drf_turbo.exceptions import ValidationError,ParseError
from drf_turbo.response import JSONResponse,UJSONResponse,ORJSONResponse,SuccessResponse,ErrorResponse
from drf_turbo.parsers import JSONParser,UJSONParser,ORJSONParser
from drf_turbo.renderers import JSONRenderer,UJSONRenderer,ORJSONRenderer

__author__ = """Michael Gendy"""
__email__ = 'mngback@gmail.com'
__version__ = '0.1.3'

__all__ = [
    'BaseSerializer',
    'Serializer',
    'ModelSerializer',
    'Field',
    'StrField',
    'EmailField',
    'URLField',
    'RegexField',
    'IPField',
    'PasswordField',
    'UUIDField',
    'SlugField',
    'IntField',
    'FloatField',
    'DecimalField',
    'BoolField',
    'ChoiceField',
    'MultipleChoiceField',
    'DateTimeField',
    'DateField',
    'TimeField',
    'FileField',
    'ArrayField',
    'DictField',
    'JSONField',
    'RelatedField',
    'ManyRelatedField',
    'ConstantField',
    'RecursiveField',
    'MethodField',
    'ValidationError',
    'ParseError',
    'JSONResponse',
    'UJSONResponse',
    'ORJSONResponse',
    'SuccessResponse',
    'ErrorResponse',
    'JSONParser',
    'UJSONParser',
    'ORJSONParser',
    'JSONRenderer',
    'UJSONRenderer',
    'ORJSONRenderer',

]








#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages
from distutils.extension import Extension
from Cython.Build import cythonize


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as requirements_file:
    requirements = requirements_file.read().splitlines()


setup(
    author="Michael Gendy",
    author_email='mngback@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="An alternative serializer implementation for REST framework written in cython built for speed.",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='drf_turbo',
    name='drf-turbo',
    packages=find_packages(include=['drf_turbo', 'drf_turbo.*']),
    test_suite='tests',
    url='https://github.com/Mng-dev-ai/drf-turbo',
    version='0.1.3',
    zip_safe=False,
    ext_modules=cythonize(["drf_turbo/*.pyx"]),

)
